@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2010 OpenStack LLC.
+# Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,21 +15,36 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+
 import os
 import re
 import urlparse
 
+from oslo.config import cfg
 import webob
 
 from cinder.api.openstack import wsgi
 from cinder.api import xmlutil
-from cinder import flags
 from cinder.openstack.common import log as logging
 from cinder import utils
 
 
+api_common_opts = [
+    cfg.IntOpt('osapi_max_limit',
+               default=1000,
+               help='the maximum number of items returned in a single '
+                    'response from a collection resource'),
+    cfg.StrOpt('osapi_volume_base_URL',
+               default=None,
+               help='Base URL that will be presented to users in links '
+                    'to the OpenStack Volume API',
+               deprecated_name='osapi_compute_link_prefix'),
+]
+
+CONF = cfg.CONF
+CONF.register_opts(api_common_opts)
+
 LOG = logging.getLogger(__name__)
-FLAGS = flags.FLAGS
 
 
 XML_NS_V1 = 'http://docs.openstack.org/volume/api/v1'
@@ -73,7 +88,7 @@ def _get_marker_param(request):
     return request.GET['marker']
 
 
-def limited(items, request, max_limit=FLAGS.osapi_max_limit):
+def limited(items, request, max_limit=CONF.osapi_max_limit):
     """Return a slice of items according to requested offset and limit.
 
     :param items: A sliceable entity
@@ -110,7 +125,7 @@ def limited(items, request, max_limit=FLAGS.osapi_max_limit):
     return items[offset:range_end]
 
 
-def limited_by_marker(items, request, max_limit=FLAGS.osapi_max_limit):
+def limited_by_marker(items, request, max_limit=CONF.osapi_max_limit):
     """Return a slice of items according to the requested marker and limit."""
     params = get_pagination_params(request)
 
@@ -192,7 +207,7 @@ class ViewBuilder(object):
         params = request.params.copy()
         params["marker"] = identifier
         prefix = self._update_link_prefix(request.application_url,
-                                          FLAGS.osapi_volume_base_URL)
+                                          CONF.osapi_volume_base_URL)
         url = os.path.join(prefix,
                            request.environ["cinder.context"].project_id,
                            self._collection_name)
@@ -201,7 +216,7 @@ class ViewBuilder(object):
     def _get_href_link(self, request, identifier):
         """Return an href string pointing to this object."""
         prefix = self._update_link_prefix(request.application_url,
-                                          FLAGS.osapi_volume_base_URL)
+                                          CONF.osapi_volume_base_URL)
         return os.path.join(prefix,
                             request.environ["cinder.context"].project_id,
                             self._collection_name,
@@ -211,7 +226,7 @@ class ViewBuilder(object):
         """Create a URL that refers to a specific resource."""
         base_url = remove_version_from_href(request.application_url)
         base_url = self._update_link_prefix(base_url,
-                                            FLAGS.osapi_volume_base_URL)
+                                            CONF.osapi_volume_base_URL)
         return os.path.join(base_url,
                             request.environ["cinder.context"].project_id,
                             self._collection_name,
