@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright (c) 2012 NTT.
 # Copyright (c) 2012 OpenStack Foundation
 # All Rights Reserved.
@@ -21,8 +23,9 @@ from cinder.api import extensions
 
 
 class Image_create(extensions.ExtensionDescriptor):
-    """Allow creating a volume from an image in the Create Volume v1 API."""
+    """Allow creating a volume from an image in the Create Volume v1 API"""
 
     name = "CreateVolumeExtension"
     alias = "os-image-create"
+    namespace = "http://docs.openstack.org/volume/ext/image-create/api/v1"
     updated = "2012-08-13T00:00:00+00:00"
