@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright (c) 2013 eBay Inc.
 # Copyright (c) 2013 OpenStack Foundation
 #
@@ -14,33 +16,66 @@
 #    under the License.
 
 """The QoS specs extension"""
-from http import HTTPStatus
 
-from oslo_log import log as logging
-from oslo_utils import timeutils
 import webob
 
-from cinder.api import api_utils
-from cinder.api import common
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
-from cinder.api.schemas import qos_specs as qos_specs_schema
-from cinder.api import validation
 from cinder.api.views import qos_specs as view_qos_specs
+from cinder.api import xmlutil
 from cinder import exception
-from cinder.i18n import _
-from cinder.policies import qos_specs as policy
-from cinder import rpc
-from cinder import utils
+from cinder.openstack.common import log as logging
+from cinder.openstack.common.notifier import api as notifier_api
+from cinder.openstack.common import strutils
 from cinder.volume import qos_specs
 
 
 LOG = logging.getLogger(__name__)
 
+authorize = extensions.extension_authorizer('volume', 'qos_specs_manage')
+
+
+def make_qos_specs(elem):
+    elem.set('id')
+    elem.set('name')
+    elem.set('consumer')
+    elem.append(SpecsTemplate())
+
+
+def make_associations(elem):
+    elem.set('association_type')
+    elem.set('name')
+    elem.set('id')
+
+
+class SpecsTemplate(xmlutil.TemplateBuilder):
+    def construct(self):
+        return xmlutil.MasterTemplate(xmlutil.make_flat_dict('specs'), 1)
+
+
+class QoSSpecsTemplate(xmlutil.TemplateBuilder):
+    def construct(self):
+        root = xmlutil.TemplateElement('qos_specs')
+        elem = xmlutil.SubTemplateElement(root, 'qos_spec',
+                                          selector='qos_specs')
+        make_qos_specs(elem)
+        return xmlutil.MasterTemplate(root, 1)
+
+
+class AssociationsTemplate(xmlutil.TemplateBuilder):
+    def construct(self):
+        root = xmlutil.TemplateElement('qos_associations')
+        elem = xmlutil.SubTemplateElement(root, 'associations',
+                                          selector='qos_associations')
+        make_associations(elem)
+        return xmlutil.MasterTemplate(root, 1)
+
 
 def _check_specs(context, specs_id):
-    # Not found exception will be handled at the wsgi level
-    qos_specs.get_qos_specs(context, specs_id)
+    try:
+        qos_specs.get_qos_specs(context, specs_id)
+    except exception.NotFound as ex:
+        raise webob.exc.HTTPNotFound(explanation=unicode(ex))
 
 
 class QoSSpecsController(wsgi.Controller):
@@ -49,143 +84,137 @@ class QoSSpecsController(wsgi.Controller):
     _view_builder_class = view_qos_specs.ViewBuilder
 
     @staticmethod
-    @utils.if_notifications_enabled
     def _notify_qos_specs_error(context, method, payload):
-        rpc.get_notifier('QoSSpecs').error(context,
-                                           method,
-                                           payload)
+        notifier_api.notify(context,
+                            'QoSSpecs',
+                            method,
+                            notifier_api.ERROR,
+                            payload)
 
+    @wsgi.serializers(xml=QoSSpecsTemplate)
     def index(self, req):
         """Returns the list of qos_specs."""
         context = req.environ['cinder.context']
-        context.authorize(policy.GET_ALL_POLICY)
-
-        params = req.params.copy()
-
-        marker, limit, offset = common.get_pagination_params(params)
-        sort_keys, sort_dirs = common.get_sort_params(params)
-        filters = params
-        allowed_search_options = ('id', 'name', 'consumer')
-        api_utils.remove_invalid_filter_options(context, filters,
-                                                allowed_search_options)
-
-        specs = qos_specs.get_all_specs(context, filters=filters,
-                                        marker=marker, limit=limit,
-                                        offset=offset, sort_keys=sort_keys,
-                                        sort_dirs=sort_dirs)
+        authorize(context)
+        specs = qos_specs.get_all_specs(context)
         return self._view_builder.summary_list(req, specs)
 
-    @validation.schema(qos_specs_schema.create)
+    @wsgi.serializers(xml=QoSSpecsTemplate)
     def create(self, req, body=None):
         context = req.environ['cinder.context']
-        context.authorize(policy.CREATE_POLICY)
+        authorize(context)
+
+        if not self.is_valid_body(body, 'qos_specs'):
+            raise webob.exc.HTTPBadRequest()
 
         specs = body['qos_specs']
-        name = specs.pop('name', None)
-        name = name.strip()
+        name = specs.get('name', None)
+        if name is None or name == "":
+            msg = _("Please specify a name for QoS specs.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
         try:
-            spec = qos_specs.create(context, name, specs)
-            notifier_info = dict(name=name,
-                                 created_at=spec.created_at,
-                                 specs=specs)
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.create',
-                                              notifier_info)
-        except exception.InvalidQoSSpecs as err:
-            notifier_err = dict(name=name, error_message=err)
+            qos_specs.create(context, name, specs)
+            spec = qos_specs.get_qos_specs_by_name(context, name)
+            notifier_info = dict(name=name, specs=specs)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'QoSSpecs.create',
+                                notifier_api.INFO, notifier_info)
+        except exception.InvalidInput as err:
+            notifier_err = dict(name=name, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.create',
                                          notifier_err)
             raise webob.exc.HTTPBadRequest(explanation=str(err))
         except exception.QoSSpecsExists as err:
-            notifier_err = dict(name=name, error_message=err)
+            notifier_err = dict(name=name, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.create',
                                          notifier_err)
             raise webob.exc.HTTPConflict(explanation=str(err))
         except exception.QoSSpecsCreateFailed as err:
-            notifier_err = dict(name=name, error_message=err)
+            notifier_err = dict(name=name, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.create',
                                          notifier_err)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=str(err))
+            raise webob.exc.HTTPInternalServerError(explanation=str(err))
 
         return self._view_builder.detail(req, spec)
 
-    @validation.schema(qos_specs_schema.set)
+    @wsgi.serializers(xml=QoSSpecsTemplate)
     def update(self, req, id, body=None):
         context = req.environ['cinder.context']
-        context.authorize(policy.UPDATE_POLICY)
+        authorize(context)
 
+        if not self.is_valid_body(body, 'qos_specs'):
+            raise webob.exc.HTTPBadRequest()
         specs = body['qos_specs']
         try:
-            spec = qos_specs.get_qos_specs(context, id)
-
             qos_specs.update(context, id, specs)
-            notifier_info = dict(id=id,
-                                 created_at=spec.created_at,
-                                 updated_at=timeutils.utcnow(),
-                                 specs=specs)
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.update',
-                                              notifier_info)
-        except (exception.QoSSpecsNotFound, exception.InvalidQoSSpecs) as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_info = dict(id=id, specs=specs)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.update',
+                                notifier_api.INFO, notifier_info)
+        except exception.QoSSpecsNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.update',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
+        except exception.InvalidQoSSpecs as err:
+            notifier_err = dict(id=id, error_message=str(err))
+            self._notify_qos_specs_error(context,
+                                         'qos_specs.update',
+                                         notifier_err)
+            raise webob.exc.HTTPBadRequest(explanation=str(err))
         except exception.QoSSpecsUpdateFailed as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.update',
                                          notifier_err)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=str(err))
+            raise webob.exc.HTTPInternalServerError(explanation=str(err))
 
         return body
 
+    @wsgi.serializers(xml=QoSSpecsTemplate)
     def show(self, req, id):
         """Return a single qos spec item."""
         context = req.environ['cinder.context']
-        context.authorize(policy.GET_POLICY)
+        authorize(context)
 
-        # Not found exception will be handled at the wsgi level
-        spec = qos_specs.get_qos_specs(context, id)
+        try:
+            spec = qos_specs.get_qos_specs(context, id)
+        except exception.QoSSpecsNotFound as err:
+            raise webob.exc.HTTPNotFound(explanation=str(err))
 
         return self._view_builder.detail(req, spec)
 
     def delete(self, req, id):
         """Deletes an existing qos specs."""
         context = req.environ['cinder.context']
-        context.authorize(policy.DELETE_POLICY)
+        authorize(context)
 
-        # Convert string to bool type in strict manner
-        force = utils.get_bool_param('force', req.params)
-        LOG.debug("Delete qos_spec: %(id)s, force: %(force)s",
+        force = req.params.get('force', None)
+
+        #convert string to bool type in strict manner
+        force = strutils.bool_from_string(force)
+        LOG.debug("Delete qos_spec: %(id)s, force: %(force)s" %
                   {'id': id, 'force': force})
-        try:
-            spec = qos_specs.get_qos_specs(context, id)
 
+        try:
             qos_specs.delete(context, id, force)
-            notifier_info = dict(id=id,
-                                 created_at=spec.created_at,
-                                 deleted_at=timeutils.utcnow())
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.delete',
-                                              notifier_info)
+            notifier_info = dict(id=id)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.delete',
+                                notifier_api.INFO, notifier_info)
         except exception.QoSSpecsNotFound as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.delete',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
         except exception.QoSSpecsInUse as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.delete',
                                          notifier_err)
@@ -195,73 +224,75 @@ class QoSSpecsController(wsgi.Controller):
             msg = _('Qos specs still in use.')
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        return webob.Response(status_int=HTTPStatus.ACCEPTED)
+        return webob.Response(status_int=202)
 
-    @validation.schema(qos_specs_schema.unset)
     def delete_keys(self, req, id, body):
         """Deletes specified keys in qos specs."""
         context = req.environ['cinder.context']
-        context.authorize(policy.DELETE_POLICY)
+        authorize(context)
+
+        if not (body and 'keys' in body
+                and isinstance(body.get('keys'), list)):
+            raise webob.exc.HTTPBadRequest()
 
         keys = body['keys']
-        LOG.debug("Delete_key spec: %(id)s, keys: %(keys)s",
+        LOG.debug("Delete_key spec: %(id)s, keys: %(keys)s" %
                   {'id': id, 'keys': keys})
 
         try:
             qos_specs.delete_keys(context, id, keys)
-            spec = qos_specs.get_qos_specs(context, id)
-            notifier_info = dict(id=id,
-                                 created_at=spec.created_at,
-                                 updated_at=spec.updated_at)
-            rpc.get_notifier('QoSSpecs').info(context, 'qos_specs.delete_keys',
-                                              notifier_info)
-        except exception.NotFound as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_info = dict(id=id)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.delete_keys',
+                                notifier_api.INFO, notifier_info)
+        except exception.QoSSpecsNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.delete_keys',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
+        except exception.QoSSpecsKeyNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
+            self._notify_qos_specs_error(context,
+                                         'qos_specs.delete_keys',
+                                         notifier_err)
+            raise webob.exc.HTTPBadRequest(explanation=str(err))
 
-        return webob.Response(status_int=HTTPStatus.ACCEPTED)
+        return webob.Response(status_int=202)
 
+    @wsgi.serializers(xml=AssociationsTemplate)
     def associations(self, req, id):
         """List all associations of given qos specs."""
         context = req.environ['cinder.context']
-        context.authorize(policy.GET_ALL_POLICY)
+        authorize(context)
 
-        LOG.debug("Get associations for qos_spec id: %s", id)
+        LOG.debug("Get associations for qos_spec id: %s" % id)
 
         try:
-            spec = qos_specs.get_qos_specs(context, id)
-
             associates = qos_specs.get_associations(context, id)
-            notifier_info = dict(id=id,
-                                 created_at=spec.created_at)
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.associations',
-                                              notifier_info)
+            notifier_info = dict(id=id)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.associations',
+                                notifier_api.INFO, notifier_info)
         except exception.QoSSpecsNotFound as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.associations',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
         except exception.CinderException as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.associations',
                                          notifier_err)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=str(err))
+            raise webob.exc.HTTPInternalServerError(explanation=str(err))
 
         return self._view_builder.associations(req, associates)
 
     def associate(self, req, id):
         """Associate a qos specs with a volume type."""
         context = req.environ['cinder.context']
-        context.authorize(policy.UPDATE_POLICY)
+        authorize(context)
 
         type_id = req.params.get('vol_type_id', None)
 
@@ -272,27 +303,29 @@ class QoSSpecsController(wsgi.Controller):
                                          'qos_specs.delete',
                                          notifier_err)
             raise webob.exc.HTTPBadRequest(explanation=msg)
-        LOG.debug("Associate qos_spec: %(id)s with type: %(type_id)s",
+        LOG.debug("Associate qos_spec: %(id)s with type: %(type_id)s" %
                   {'id': id, 'type_id': type_id})
 
         try:
-            spec = qos_specs.get_qos_specs(context, id)
-
             qos_specs.associate_qos_with_type(context, id, type_id)
-            notifier_info = dict(id=id, type_id=type_id,
-                                 created_at=spec.created_at)
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.associate',
-                                              notifier_info)
-        except exception.NotFound as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_info = dict(id=id, type_id=type_id)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.associate',
+                                notifier_api.INFO, notifier_info)
+        except exception.VolumeTypeNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.associate',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
+        except exception.QoSSpecsNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
+            self._notify_qos_specs_error(context,
+                                         'qos_specs.associate',
+                                         notifier_err)
+            raise webob.exc.HTTPNotFound(explanation=str(err))
         except exception.InvalidVolumeType as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.associate',
                                          notifier_err)
@@ -301,19 +334,18 @@ class QoSSpecsController(wsgi.Controller):
                                          notifier_err)
             raise webob.exc.HTTPBadRequest(explanation=str(err))
         except exception.QoSSpecsAssociateFailed as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.associate',
                                          notifier_err)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=str(err))
+            raise webob.exc.HTTPInternalServerError(explanation=str(err))
 
-        return webob.Response(status_int=HTTPStatus.ACCEPTED)
+        return webob.Response(status_int=202)
 
     def disassociate(self, req, id):
         """Disassociate a qos specs from a volume type."""
         context = req.environ['cinder.context']
-        context.authorize(policy.UPDATE_POLICY)
+        authorize(context)
 
         type_id = req.params.get('vol_type_id', None)
 
@@ -324,74 +356,71 @@ class QoSSpecsController(wsgi.Controller):
                                          'qos_specs.delete',
                                          notifier_err)
             raise webob.exc.HTTPBadRequest(explanation=msg)
-        LOG.debug("Disassociate qos_spec: %(id)s from type: %(type_id)s",
+        LOG.debug("Disassociate qos_spec: %(id)s from type: %(type_id)s" %
                   {'id': id, 'type_id': type_id})
 
         try:
-            spec = qos_specs.get_qos_specs(context, id)
-
             qos_specs.disassociate_qos_specs(context, id, type_id)
-            notifier_info = dict(id=id, type_id=type_id,
-                                 created_at=spec.created_at)
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.disassociate',
-                                              notifier_info)
-        except exception.NotFound as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_info = dict(id=id, type_id=type_id)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.disassociate',
+                                notifier_api.INFO, notifier_info)
+        except exception.VolumeTypeNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.disassociate',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
+        except exception.QoSSpecsNotFound as err:
+            notifier_err = dict(id=id, error_message=str(err))
+            self._notify_qos_specs_error(context,
+                                         'qos_specs.disassociate',
+                                         notifier_err)
+            raise webob.exc.HTTPNotFound(explanation=str(err))
         except exception.QoSSpecsDisassociateFailed as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.disassociate',
                                          notifier_err)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=str(err))
+            raise webob.exc.HTTPInternalServerError(explanation=str(err))
 
-        return webob.Response(status_int=HTTPStatus.ACCEPTED)
+        return webob.Response(status_int=202)
 
     def disassociate_all(self, req, id):
         """Disassociate a qos specs from all volume types."""
         context = req.environ['cinder.context']
-        context.authorize(policy.UPDATE_POLICY)
+        authorize(context)
 
-        LOG.debug("Disassociate qos_spec: %s from all.", id)
+        LOG.debug("Disassociate qos_spec: %s from all." % id)
 
         try:
-            spec = qos_specs.get_qos_specs(context, id)
-
             qos_specs.disassociate_all(context, id)
-            notifier_info = dict(id=id,
-                                 created_at=spec.created_at)
-            rpc.get_notifier('QoSSpecs').info(context,
-                                              'qos_specs.disassociate_all',
-                                              notifier_info)
+            notifier_info = dict(id=id)
+            notifier_api.notify(context, 'QoSSpecs',
+                                'qos_specs.disassociate_all',
+                                notifier_api.INFO, notifier_info)
         except exception.QoSSpecsNotFound as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.disassociate_all',
                                          notifier_err)
-            # Not found exception will be handled at the wsgi level
-            raise
+            raise webob.exc.HTTPNotFound(explanation=str(err))
         except exception.QoSSpecsDisassociateFailed as err:
-            notifier_err = dict(id=id, error_message=err)
+            notifier_err = dict(id=id, error_message=str(err))
             self._notify_qos_specs_error(context,
                                          'qos_specs.disassociate_all',
                                          notifier_err)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=str(err))
+            raise webob.exc.HTTPInternalServerError(explanation=str(err))
 
-        return webob.Response(status_int=HTTPStatus.ACCEPTED)
+        return webob.Response(status_int=202)
 
 
 class Qos_specs_manage(extensions.ExtensionDescriptor):
-    """QoS specs support."""
+    """QoS specs support"""
 
     name = "Qos_specs_manage"
     alias = "qos-specs"
+    namespace = "http://docs.openstack.org/volume/ext/qos-specs/api/v1"
     updated = "2013-08-02T00:00:00+00:00"
 
     def get_resources(self):
