@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,6 +23,7 @@ from cinder.api import xmlutil
 from cinder import db
 from cinder.db.sqlalchemy import api as sqlalchemy_api
 from cinder import exception
+from cinder.openstack.common import strutils
 from cinder import quota
 
 
@@ -45,24 +46,29 @@ class QuotaTemplate(xmlutil.TemplateBuilder):
         return xmlutil.MasterTemplate(root, 1)
 
 
-class QuotaSetsController(object):
+class QuotaSetsController(wsgi.Controller):
 
     def _format_quota_set(self, project_id, quota_set):
         """Convert the quota object to a result dict"""
 
-        result = dict(id=str(project_id))
+        quota_set['id'] = str(project_id)
 
-        for resource in QUOTAS.resources:
-            result[resource] = quota_set[resource]
-
-        return dict(quota_set=result)
+        return dict(quota_set=quota_set)
 
     def _validate_quota_limit(self, limit):
+        try:
+            limit = int(limit)
+        except ValueError:
+            msg = _("Quota limit must be specified as an integer value.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         # NOTE: -1 is a flag value for unlimited
         if limit < -1:
             msg = _("Quota limit must be -1 or greater.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
+        return limit
+
     def _get_quotas(self, context, id, usages=False):
         values = QUOTAS.get_project_quotas(context, id, usages=usages)
 
@@ -75,22 +81,32 @@ class QuotaSetsController(object):
     def show(self, req, id):
         context = req.environ['cinder.context']
         authorize_show(context)
+
+        params = req.params
+        if not hasattr(params, '__call__') and 'usage' in params:
+            usage = strutils.bool_from_string(params['usage'])
+        else:
+            usage = False
+
         try:
             sqlalchemy_api.authorize_project_context(context, id)
         except exception.NotAuthorized:
             raise webob.exc.HTTPForbidden()
 
-        return self._format_quota_set(id, self._get_quotas(context, id))
+        return self._format_quota_set(id, self._get_quotas(context, id, usage))
 
     @wsgi.serializers(xml=QuotaTemplate)
     def update(self, req, id, body):
         context = req.environ['cinder.context']
         authorize_update(context)
         project_id = id
+        if not self.is_valid_body(body, 'quota_set'):
+            msg = _("Missing required element quota_set in request body.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         for key in body['quota_set'].keys():
             if key in QUOTAS:
-                value = int(body['quota_set'][key])
-                self._validate_quota_limit(value)
+                value = self._validate_quota_limit(body['quota_set'][key])
                 try:
                     db.quota_update(context, project_id, key, value)
                 except exception.ProjectQuotaNotFound:
