@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,7 +18,10 @@ import webob.exc
 
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
-from cinder.i18n import _
+from cinder.api.v2 import volumes
+from cinder.openstack.common import log as logging
+
+LOG = logging.getLogger(__name__)
 
 
 class SchedulerHintsController(wsgi.Controller):
@@ -31,7 +36,7 @@ class SchedulerHintsController(wsgi.Controller):
                 hints.update(body[attr])
         except ValueError:
             msg = _("Malformed scheduler_hints attribute")
-            raise webob.exc.HTTPBadRequest(explanation=msg)
+            raise webob.exc.HTTPBadRequest(reason=msg)
 
         return hints
 
@@ -49,6 +54,7 @@ class Scheduler_hints(extensions.ExtensionDescriptor):
 
     name = "SchedulerHints"
     alias = "OS-SCH-HNT"
+    namespace = volumes.SCHEDULER_HINTS_NAMESPACE
     updated = "2013-04-18T00:00:00+00:00"
 
     def get_controller_extensions(self):
