@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 IBM
+# Copyright 2012 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,6 +16,7 @@
 #    under the License.
 
 
+from oslo.config import cfg
 import webob.exc
 
 from cinder.api import extensions
@@ -28,6 +29,8 @@ from cinder.openstack.common import timeutils
 from cinder import utils
 
 
+CONF = cfg.CONF
+
 LOG = logging.getLogger(__name__)
 authorize = extensions.extension_authorizer('volume', 'services')
 
@@ -48,10 +51,16 @@ class ServicesIndexTemplate(xmlutil.TemplateBuilder):
 
 class ServicesUpdateTemplate(xmlutil.TemplateBuilder):
     def construct(self):
+        # TODO(uni): template elements of 'host', 'service' and 'disabled'
+        # should be deprecated to make ServicesUpdateTemplate consistent
+        # with ServicesIndexTemplate. Still keeping it here for API
+        # compatibility sake.
         root = xmlutil.TemplateElement('host')
         root.set('host')
         root.set('service')
         root.set('disabled')
+        root.set('binary')
+        root.set('status')
 
         return xmlutil.MasterTemplate(root, 1)
 
@@ -73,15 +82,23 @@ class ServiceController(object):
         service = ''
         if 'service' in req.GET:
             service = req.GET['service']
+            LOG.deprecated(_("Query by service parameter is deprecated. "
+                             "Please use binary parameter instead."))
+        binary = ''
+        if 'binary' in req.GET:
+            binary = req.GET['binary']
+
         if host:
             services = [s for s in services if s['host'] == host]
-        if service:
-            services = [s for s in services if s['binary'] == service]
+        # NOTE(uni): deprecating service request key, binary takes precedence
+        binary_key = binary or service
+        if binary_key:
+            services = [s for s in services if s['binary'] == binary_key]
 
         svcs = []
         for svc in services:
             delta = now - (svc['updated_at'] or svc['created_at'])
-            alive = abs(utils.total_seconds(delta))
+            alive = abs(utils.total_seconds(delta)) <= CONF.service_down_time
             art = (alive and "up") or "down"
             active = 'enabled'
             if svc['disabled']:
@@ -107,12 +124,19 @@ class ServiceController(object):
 
         try:
             host = body['host']
-            service = body['service']
         except (TypeError, KeyError):
             raise webob.exc.HTTPBadRequest()
 
+        # NOTE(uni): deprecating service request key, binary takes precedence
+        # Still keeping service key here for API compatibility sake.
+        service = body.get('service', '')
+        binary = body.get('binary', '')
+        binary_key = binary or service
+        if not binary_key:
+            raise webob.exc.HTTPBadRequest()
+
         try:
-            svc = db.service_get_by_args(context, host, service)
+            svc = db.service_get_by_args(context, host, binary_key)
             if not svc:
                 raise webob.exc.HTTPNotFound('Unknown service')
 
@@ -120,7 +144,12 @@ class ServiceController(object):
         except exception.ServiceNotFound:
             raise webob.exc.HTTPNotFound("service not found")
 
-        return {'host': host, 'service': service, 'disabled': disabled}
+        status = id + 'd'
+        return {'host': host,
+                'service': service,
+                'disabled': disabled,
+                'binary': binary,
+                'status': status}
 
 
 class Services(extensions.ExtensionDescriptor):
