@@ -12,19 +12,22 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-from oslo_log import log as logging
-from six.moves import http_client
+from oslo.config import cfg
 import webob
 
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
-from cinder.i18n import _
-from cinder import objects
-from cinder.objects import fields
-from cinder.policies import snapshot_actions as policy
+from cinder import db
+from cinder.openstack.common import log as logging
+
 LOG = logging.getLogger(__name__)
 
 
+def authorize(context, action_name):
+    action = 'snapshot_actions:%s' % action_name
+    extensions.extension_authorizer('snapshot', action)(context)
+
+
 class SnapshotActionsController(wsgi.Controller):
     def __init__(self, *args, **kwargs):
         super(SnapshotActionsController, self).__init__(*args, **kwargs)
@@ -40,37 +43,28 @@ class SnapshotActionsController(wsgi.Controller):
         """
 
         context = req.environ['cinder.context']
-        LOG.debug("body: %s", body)
-        try:
-            status = body['os-update_snapshot_status']['status']
-        except KeyError:
-            msg = _("'status' must be specified.")
-            raise webob.exc.HTTPBadRequest(explanation=msg)
+        authorize(context, 'update_snapshot_status')
+
+        LOG.debug("body: %s" % body)
+        status = body['os-update_snapshot_status']['status']
 
         # Allowed state transitions
-        status_map = {fields.SnapshotStatus.CREATING:
-                      [fields.SnapshotStatus.CREATING,
-                       fields.SnapshotStatus.AVAILABLE,
-                       fields.SnapshotStatus.ERROR],
-                      fields.SnapshotStatus.DELETING:
-                      [fields.SnapshotStatus.DELETING,
-                       fields.SnapshotStatus.ERROR_DELETING]}
+        status_map = {'creating': ['creating', 'available', 'error'],
+                      'deleting': ['deleting', 'error_deleting']}
 
-        current_snapshot = objects.Snapshot.get_by_id(context, id)
-        context.authorize(policy.UPDATE_STATUS_POLICY,
-                          target_obj=current_snapshot)
+        current_snapshot = db.snapshot_get(context, id)
 
-        if current_snapshot.status not in status_map:
+        if current_snapshot['status'] not in status_map:
             msg = _("Snapshot status %(cur)s not allowed for "
                     "update_snapshot_status") % {
-                        'cur': current_snapshot.status}
+                        'cur': current_snapshot['status']}
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        if status not in status_map[current_snapshot.status]:
+        if status not in status_map[current_snapshot['status']]:
             msg = _("Provided snapshot status %(provided)s not allowed for "
                     "snapshot with status %(current)s.") % \
                 {'provided': status,
-                 'current': current_snapshot.status}
+                 'current': current_snapshot['status']}
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         update_dict = {'id': id,
@@ -89,12 +83,11 @@ class SnapshotActionsController(wsgi.Controller):
 
             update_dict.update({'progress': progress})
 
-        LOG.info("Updating snapshot %(id)s with info %(dict)s",
+        LOG.info("Updating snapshot %(id)s with info %(dict)s" %
                  {'id': id, 'dict': update_dict})
 
-        current_snapshot.update(update_dict)
-        current_snapshot.save()
-        return webob.Response(status_int=http_client.ACCEPTED)
+        db.snapshot_update(context, id, update_dict)
+        return webob.Response(status_int=202)
 
 
 class Snapshot_actions(extensions.ExtensionDescriptor):
@@ -102,6 +95,8 @@ class Snapshot_actions(extensions.ExtensionDescriptor):
 
     name = "SnapshotActions"
     alias = "os-snapshot-actions"
+    namespace = \
+        "http://docs.openstack.org/volume/ext/snapshot-actions/api/v1.1"
     updated = "2013-07-16T00:00:00+00:00"
 
     def get_controller_extensions(self):
