@@ -1,4 +1,4 @@
-#   Copyright 2012 OpenStack, LLC.
+#   Copyright 2012 OpenStack Foundation
 #
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
@@ -18,14 +18,13 @@ from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder.api import xmlutil
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
 from cinder.openstack.common.rpc import common as rpc_common
+from cinder.openstack.common import strutils
 from cinder import utils
 from cinder import volume
 
 
-FLAGS = flags.FLAGS
 LOG = logging.getLogger(__name__)
 
 
@@ -78,12 +77,39 @@ class VolumeActionsController(wsgi.Controller):
         """Add attachment metadata."""
         context = req.environ['cinder.context']
         volume = self.volume_api.get(context, id)
-
-        instance_uuid = body['os-attach']['instance_uuid']
+        # instance uuid is an option now
+        instance_uuid = None
+        if 'instance_uuid' in body['os-attach']:
+            instance_uuid = body['os-attach']['instance_uuid']
+        host_name = None
+        # Keep API backward compatibility
+        if 'host_name' in body['os-attach']:
+            host_name = body['os-attach']['host_name']
         mountpoint = body['os-attach']['mountpoint']
+        if 'mode' in body['os-attach']:
+            mode = body['os-attach']['mode']
+        else:
+            mode = 'rw'
+
+        if instance_uuid and host_name:
+            msg = _("Invalid request to attach volume to an "
+                    "instance %(instance_uuid)s and a "
+                    "host %(host_name)s simultaneously") % {
+                        'instance_uuid': instance_uuid,
+                        'host_name': host_name,
+                    }
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        elif instance_uuid is None and host_name is None:
+            msg = _("Invalid request to attach volume to an invalid target")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        if mode not in ('rw', 'ro'):
+            msg = _("Invalid request to attach volume with an invalid mode. "
+                    "Attaching mode should be 'rw' or 'ro'")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
         self.volume_api.attach(context, volume,
-                               instance_uuid, mountpoint)
+                               instance_uuid, host_name, mountpoint, mode)
         return webob.Response(status_int=202)
 
     @wsgi.action('os-detach')
@@ -166,8 +192,8 @@ class VolumeActionsController(wsgi.Controller):
         force = params.get('force', False)
         try:
             volume = self.volume_api.get(context, id)
-        except exception.VolumeNotFound, error:
-            raise webob.exc.HTTPNotFound(explanation=unicode(error))
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
         authorize(context, "upload_image")
         image_metadata = {"container_format": params.get("container_format",
                                                          "bare"),
@@ -178,9 +204,9 @@ class VolumeActionsController(wsgi.Controller):
                                                             volume,
                                                             image_metadata,
                                                             force)
-        except exception.InvalidVolume, error:
-            raise webob.exc.HTTPBadRequest(explanation=unicode(error))
-        except ValueError, error:
+        except exception.InvalidVolume as error:
+            raise webob.exc.HTTPBadRequest(explanation=error.msg)
+        except ValueError as error:
             raise webob.exc.HTTPBadRequest(explanation=unicode(error))
         except rpc_common.RemoteError as error:
             msg = "%(err_type)s: %(err_msg)s" % {'err_type': error.exc_type,
@@ -188,6 +214,43 @@ class VolumeActionsController(wsgi.Controller):
             raise webob.exc.HTTPBadRequest(explanation=msg)
         return {'os-volume_upload_image': response}
 
+    @wsgi.action('os-extend')
+    def _extend(self, req, id, body):
+        """Extend size of volume."""
+        context = req.environ['cinder.context']
+        volume = self.volume_api.get(context, id)
+        try:
+            _val = int(body['os-extend']['new_size'])
+        except (KeyError, ValueError):
+            msg = _("New volume size must be specified as an integer.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        size = body['os-extend']['new_size']
+        self.volume_api.extend(context, volume, size)
+        return webob.Response(status_int=202)
+
+    @wsgi.action('os-update_readonly_flag')
+    def _volume_readonly_update(self, req, id, body):
+        """Update volume readonly flag."""
+        context = req.environ['cinder.context']
+        volume = self.volume_api.get(context, id)
+
+        readonly_flag = body['os-update_readonly_flag'].get('readonly')
+        if isinstance(readonly_flag, basestring):
+            try:
+                readonly_flag = strutils.bool_from_string(readonly_flag,
+                                                          strict=True)
+            except ValueError:
+                msg = _("Bad value for 'readonly'")
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        elif not isinstance(readonly_flag, bool):
+            msg = _("'readonly' not string or bool")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        self.volume_api.update_readonly_flag(context, volume, readonly_flag)
+        return webob.Response(status_int=202)
+
 
 class Volume_actions(extensions.ExtensionDescriptor):
     """Enable volume actions
