@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2011 OpenStack Foundation
 # Copyright 2011 Justin Santa Barbara
 # All Rights Reserved.
@@ -16,22 +18,22 @@
 
 import os
 
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import importutils
+from oslo.config import cfg
 import webob.dec
 import webob.exc
 
 import cinder.api.openstack
 from cinder.api.openstack import wsgi
+from cinder.api import xmlutil
 from cinder import exception
+from cinder.openstack.common import importutils
+from cinder.openstack.common import log as logging
 import cinder.policy
 
 
 CONF = cfg.CONF
 
 LOG = logging.getLogger(__name__)
-FILES_TO_SKIP = ['resource_common_manage.py']
 
 
 class ExtensionDescriptor(object):
@@ -48,6 +50,12 @@ class ExtensionDescriptor(object):
     # The alias for the extension, e.g., 'FOXNSOX'
     alias = None
 
+    # Description comes from the docstring for the class
+
+    # The XML namespace for the extension, e.g.,
+    # 'http://www.fox.in.socks/api/ext/pie/v1.0'
+    namespace = None
+
     # The timestamp when the extension was last updated, e.g.,
     # '2011-01-22T13:25:27-06:00'
     updated = None
@@ -56,7 +64,6 @@ class ExtensionDescriptor(object):
         """Register extension with the extension manager."""
 
         ext_mgr.register(self)
-        self.ext_mgr = ext_mgr
 
     def get_resources(self):
         """List of extensions.ResourceExtension extension objects.
@@ -75,6 +82,55 @@ class ExtensionDescriptor(object):
         controller_exts = []
         return controller_exts
 
+    @classmethod
+    def nsmap(cls):
+        """Synthesize a namespace map from extension."""
+
+        # Start with a base nsmap
+        nsmap = ext_nsmap.copy()
+
+        # Add the namespace for the extension
+        nsmap[cls.alias] = cls.namespace
+
+        return nsmap
+
+    @classmethod
+    def xmlname(cls, name):
+        """Synthesize element and attribute names."""
+
+        return '{%s}%s' % (cls.namespace, name)
+
+
+def make_ext(elem):
+    elem.set('name')
+    elem.set('namespace')
+    elem.set('alias')
+    elem.set('updated')
+
+    desc = xmlutil.SubTemplateElement(elem, 'description')
+    desc.text = 'description'
+
+    xmlutil.make_links(elem, 'links')
+
+
+ext_nsmap = {None: xmlutil.XMLNS_COMMON_V10, 'atom': xmlutil.XMLNS_ATOM}
+
+
+class ExtensionTemplate(xmlutil.TemplateBuilder):
+    def construct(self):
+        root = xmlutil.TemplateElement('extension', selector='extension')
+        make_ext(root)
+        return xmlutil.MasterTemplate(root, 1, nsmap=ext_nsmap)
+
+
+class ExtensionsTemplate(xmlutil.TemplateBuilder):
+    def construct(self):
+        root = xmlutil.TemplateElement('extensions')
+        elem = xmlutil.SubTemplateElement(root, 'extension',
+                                          selector='extensions')
+        make_ext(elem)
+        return xmlutil.MasterTemplate(root, 1, nsmap=ext_nsmap)
+
 
 class ExtensionsResource(wsgi.Resource):
 
@@ -87,16 +143,19 @@ class ExtensionsResource(wsgi.Resource):
         ext_data['name'] = ext.name
         ext_data['alias'] = ext.alias
         ext_data['description'] = ext.__doc__
+        ext_data['namespace'] = ext.namespace
         ext_data['updated'] = ext.updated
         ext_data['links'] = []  # TODO(dprince): implement extension links
         return ext_data
 
+    @wsgi.serializers(xml=ExtensionsTemplate)
     def index(self, req):
         extensions = []
-        for _alias, ext in self.extension_manager.extensions.items():
+        for _alias, ext in self.extension_manager.extensions.iteritems():
             extensions.append(self._translate(ext))
         return dict(extensions=extensions)
 
+    @wsgi.serializers(xml=ExtensionTemplate)
     def show(self, req, id):
         try:
             # NOTE(dprince): the extensions alias is used as the 'id' for show
@@ -122,7 +181,7 @@ class ExtensionManager(object):
     """
 
     def __init__(self):
-        LOG.debug('Initializing extension manager.')
+        LOG.audit(_('Initializing extension manager.'))
 
         self.cls_list = CONF.osapi_volume_extension
         self.extensions = {}
@@ -137,11 +196,10 @@ class ExtensionManager(object):
             return
 
         alias = ext.alias
-        LOG.debug('Loaded extension: %s', alias)
+        LOG.audit(_('Loaded extension: %s'), alias)
 
         if alias in self.extensions:
-            raise exception.CinderException(
-                "Found duplicate extension: %s" % alias)
+            raise exception.Error("Found duplicate extension: %s" % alias)
         self.extensions[alias] = ext
 
     def get_resources(self):
@@ -176,13 +234,14 @@ class ExtensionManager(object):
     def _check_extension(self, extension):
         """Checks for required methods in extension objects."""
         try:
-            LOG.debug('Ext name: %s', extension.name)
-            LOG.debug('Ext alias: %s', extension.alias)
-            LOG.debug('Ext description: %s',
+            LOG.debug(_('Ext name: %s'), extension.name)
+            LOG.debug(_('Ext alias: %s'), extension.alias)
+            LOG.debug(_('Ext description: %s'),
                       ' '.join(extension.__doc__.strip().split()))
-            LOG.debug('Ext updated: %s', extension.updated)
-        except AttributeError:
-            LOG.exception("Exception loading extension.")
+            LOG.debug(_('Ext namespace: %s'), extension.namespace)
+            LOG.debug(_('Ext updated: %s'), extension.updated)
+        except AttributeError as ex:
+            LOG.exception(_("Exception loading extension: %s"), unicode(ex))
             return False
 
         return True
@@ -196,13 +255,13 @@ class ExtensionManager(object):
         expected to call the register() method at least once.
         """
 
-        LOG.debug("Loading extension %s", ext_factory)
+        LOG.debug(_("Loading extension %s"), ext_factory)
 
         # Load the factory
         factory = importutils.import_class(ext_factory)
 
         # Call it
-        LOG.debug("Calling extension factory %s", ext_factory)
+        LOG.debug(_("Calling extension factory %s"), ext_factory)
         factory(self)
 
     def _load_extensions(self):
@@ -210,13 +269,26 @@ class ExtensionManager(object):
 
         extensions = list(self.cls_list)
 
+        # NOTE(thingee): Backwards compat for the old extension loader path.
+        # We can drop this post-grizzly in the H release.
+        old_contrib_path = ('cinder.api.openstack.volume.contrib.'
+                            'standard_extensions')
+        new_contrib_path = 'cinder.api.contrib.standard_extensions'
+        if old_contrib_path in extensions:
+            LOG.warn(_('osapi_volume_extension is set to deprecated path: %s'),
+                     old_contrib_path)
+            LOG.warn(_('Please set your flag or cinder.conf settings for '
+                       'osapi_volume_extension to: %s'), new_contrib_path)
+            extensions = [e.replace(old_contrib_path, new_contrib_path)
+                          for e in extensions]
+
         for ext_factory in extensions:
             try:
                 self.load_extension(ext_factory)
             except Exception as exc:
-                LOG.warning('Failed to load extension %(ext_factory)s: '
-                            '%(exc)s',
-                            {'ext_factory': ext_factory, 'exc': exc})
+                LOG.warn(_('Failed to load extension %(ext_factory)s: '
+                           '%(exc)s'),
+                         {'ext_factory': ext_factory, 'exc': exc})
 
 
 class ControllerExtension(object):
@@ -263,17 +335,12 @@ def load_standard_extensions(ext_mgr, logger, path, package, ext_list=None):
         else:
             relpkg = '.%s' % '.'.join(relpath.split(os.sep))
 
-        # Now, consider each file in turn, only considering .py and .pyc files
+        # Now, consider each file in turn, only considering .py files
         for fname in filenames:
             root, ext = os.path.splitext(fname)
 
-            # Skip __init__ and anything that's not .py and .pyc
-            if ((ext not in ('.py', '.pyc')) or root == '__init__' or
-                    fname in FILES_TO_SKIP):
-                continue
-
-            # If .pyc and .py both exist, skip .pyc
-            if ext == '.pyc' and ((root + '.py') in filenames):
+            # Skip __init__ and anything that's not .py
+            if ext != '.py' or root == '__init__':
                 continue
 
             # Try loading it
@@ -282,15 +349,15 @@ def load_standard_extensions(ext_mgr, logger, path, package, ext_list=None):
                          (package, relpkg, root, classname))
 
             if ext_list is not None and classname not in ext_list:
-                logger.debug("Skipping extension: %s", classpath)
+                logger.debug("Skipping extension: %s" % classpath)
                 continue
 
             try:
                 ext_mgr.load_extension(classpath)
             except Exception as exc:
-                logger.warning('Failed to load extension %(classpath)s: '
-                               '%(exc)s',
-                               {'classpath': classpath, 'exc': exc})
+                logger.warn(_('Failed to load extension %(classpath)s: '
+                              '%(exc)s'),
+                            {'classpath': classpath, 'exc': exc})
 
         # Now, let's consider any subdirectories we may have...
         subdirs = []
@@ -313,22 +380,31 @@ def load_standard_extensions(ext_mgr, logger, path, package, ext_list=None):
                 try:
                     ext(ext_mgr)
                 except Exception as exc:
-                    logger.warning('Failed to load extension '
-                                   '%(ext_name)s: %(exc)s',
-                                   {'ext_name': ext_name, 'exc': exc})
+                    logger.warn(_('Failed to load extension %(ext_name)s: '
+                                  '%(exc)s'),
+                                {'ext_name': ext_name, 'exc': exc})
 
         # Update the list of directories we'll explore...
         dirnames[:] = subdirs
 
 
 def extension_authorizer(api_name, extension_name):
-    def authorize(context, target=None, action=None):
+    def authorize(context, target=None):
         if target is None:
             target = {'project_id': context.project_id,
                       'user_id': context.user_id}
-        if action is None:
-            act = '%s_extension:%s' % (api_name, extension_name)
-        else:
-            act = '%s_extension:%s:%s' % (api_name, extension_name, action)
-        cinder.policy.authorize(context, act, target)
+        action = '%s_extension:%s' % (api_name, extension_name)
+        cinder.policy.enforce(context, action, target)
+    return authorize
+
+
+def soft_extension_authorizer(api_name, extension_name):
+    hard_authorize = extension_authorizer(api_name, extension_name)
+
+    def authorize(context):
+        try:
+            hard_authorize(context)
+            return True
+        except exception.NotAuthorized:
+            return False
     return authorize
