@@ -23,8 +23,8 @@ from cinder.api.openstack import wsgi
 from cinder.api.v1 import volumes
 from cinder.api import xmlutil
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
+from cinder.openstack.common import strutils
 from cinder import utils
 from cinder import volume
 
@@ -32,9 +32,6 @@ from cinder import volume
 LOG = logging.getLogger(__name__)
 
 
-FLAGS = flags.FLAGS
-
-
 def _translate_snapshot_detail_view(context, snapshot):
     """Maps keys for snapshots details view."""
 
@@ -180,7 +177,7 @@ class SnapshotsController(wsgi.Controller):
             msg = _("Invalid value '%s' for force. ") % force
             raise exception.InvalidParameterValue(err=msg)
 
-        if utils.bool_from_str(force):
+        if strutils.bool_from_string(force):
             new_snapshot = self.volume_api.create_snapshot_force(
                 context,
                 volume,
