@@ -15,6 +15,7 @@
 
 """The volumes api."""
 
+import ast
 import webob
 from webob import exc
 
@@ -22,20 +23,16 @@ from cinder.api import common
 from cinder.api.openstack import wsgi
 from cinder.api import xmlutil
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import uuidutils
 from cinder import utils
-from cinder import volume
+from cinder import volume as cinder_volume
 from cinder.volume import volume_types
 
 
 LOG = logging.getLogger(__name__)
 
 
-FLAGS = flags.FLAGS
-
-
 def _translate_attachment_detail_view(_context, vol):
     """Maps keys for attachment details view."""
 
@@ -57,6 +54,7 @@ def _translate_attachment_summary_view(_context, vol):
 
     d['volume_id'] = volume_id
     d['server_id'] = vol['instance_uuid']
+    d['host_name'] = vol['attached_host']
     if vol.get('mountpoint'):
         d['device'] = vol['mountpoint']
 
@@ -83,6 +81,13 @@ def _translate_volume_summary_view(context, vol, image_id=None):
     d['availability_zone'] = vol['availability_zone']
     d['created_at'] = vol['created_at']
 
+    # Need to form the string true/false explicitly here to
+    # maintain our API contract
+    if vol['bootable']:
+        d['bootable'] = 'true'
+    else:
+        d['bootable'] = 'false'
+
     d['attachments'] = []
     if vol['attach_status'] == 'attached':
         attachment = _translate_attachment_detail_view(context, vol)
@@ -114,17 +119,13 @@ def _translate_volume_summary_view(context, vol, image_id=None):
     else:
         d['metadata'] = {}
 
-    if vol.get('volume_glance_metadata'):
-        d['bootable'] = 'true'
-    else:
-        d['bootable'] = 'false'
-
     return d
 
 
 def make_attachment(elem):
     elem.set('id')
     elem.set('server_id')
+    elem.set('host_name')
     elem.set('volume_id')
     elem.set('device')
 
@@ -136,6 +137,7 @@ def make_volume(elem):
     elem.set('availability_zone')
     elem.set('created_at')
     elem.set('display_name')
+    elem.set('bootable')
     elem.set('display_description')
     elem.set('volume_type')
     elem.set('snapshot_id')
@@ -183,7 +185,8 @@ class CommonDeserializer(wsgi.MetadataXMLDeserializer):
         volume_node = self.find_first_child_named(node, 'volume')
 
         attributes = ['display_name', 'display_description', 'size',
-                      'volume_type', 'availability_zone']
+                      'volume_type', 'availability_zone', 'imageRef',
+                      'snapshot_id', 'source_volid']
         for attr in attributes:
             if volume_node.getAttribute(attr):
                 volume[attr] = volume_node.getAttribute(attr)
@@ -212,11 +215,59 @@ class CreateDeserializer(CommonDeserializer):
 class VolumeController(wsgi.Controller):
     """The Volumes API controller for the OpenStack API."""
 
+    _visible_admin_metadata_keys = ['readonly', 'attached_mode']
+
     def __init__(self, ext_mgr):
-        self.volume_api = volume.API()
+        self.volume_api = cinder_volume.API()
         self.ext_mgr = ext_mgr
         super(VolumeController, self).__init__()
 
+    def _add_visible_admin_metadata(self, context, volume):
+        if context is None:
+            return
+
+        visible_admin_meta = {}
+
+        if context.is_admin:
+            volume_tmp = volume
+        else:
+            try:
+                volume_tmp = self.volume_api.get(context.elevated(),
+                                                 volume['id'])
+            except Exception:
+                return
+
+        if volume_tmp.get('volume_admin_metadata'):
+            for item in volume_tmp['volume_admin_metadata']:
+                if item['key'] in self._visible_admin_metadata_keys:
+                    visible_admin_meta[item['key']] = item['value']
+        # avoid circular ref when volume is a Volume instance
+        elif (volume_tmp.get('admin_metadata') and
+                isinstance(volume_tmp.get('admin_metadata'), dict)):
+            for key in self._visible_admin_metadata_keys:
+                if key in volume_tmp['admin_metadata'].keys():
+                    visible_admin_meta[key] = volume_tmp['admin_metadata'][key]
+
+        if not visible_admin_meta:
+            return
+
+        # NOTE(zhiyan): update visible administration metadata to
+        # volume metadata, administration metadata will rewrite existing key.
+        if volume.get('volume_metadata'):
+            orig_meta = list(volume.get('volume_metadata'))
+            for item in orig_meta:
+                if item['key'] in visible_admin_meta.keys():
+                    item['value'] = visible_admin_meta.pop(item['key'])
+            for key, value in visible_admin_meta.iteritems():
+                orig_meta.append({'key': key, 'value': value})
+            volume['volume_metadata'] = orig_meta
+        # avoid circular ref when vol is a Volume instance
+        elif (volume.get('metadata') and
+                isinstance(volume.get('metadata'), dict)):
+            volume['metadata'].update(visible_admin_meta)
+        else:
+            volume['metadata'] = visible_admin_meta
+
     @wsgi.serializers(xml=VolumeTemplate)
     def show(self, req, id):
         """Return data about the given volume."""
@@ -224,9 +275,12 @@ class VolumeController(wsgi.Controller):
 
         try:
             vol = self.volume_api.get(context, id)
+            req.cache_resource(vol)
         except exception.NotFound:
             raise exc.HTTPNotFound()
 
+        self._add_visible_admin_metadata(context, vol)
+
         return {'volume': _translate_volume_detail_view(context, vol)}
 
     def delete(self, req, id):
@@ -260,6 +314,9 @@ class VolumeController(wsgi.Controller):
         search_opts.pop('limit', None)
         search_opts.pop('offset', None)
 
+        if 'metadata' in search_opts:
+            search_opts['metadata'] = ast.literal_eval(search_opts['metadata'])
+
         context = req.environ['cinder.context']
         remove_invalid_options(context,
                                search_opts, self._get_volume_search_options())
@@ -267,7 +324,14 @@ class VolumeController(wsgi.Controller):
         volumes = self.volume_api.get_all(context, marker=None, limit=None,
                                           sort_key='created_at',
                                           sort_dir='desc', filters=search_opts)
+
+        volumes = [dict(vol.iteritems()) for vol in volumes]
+
+        for volume in volumes:
+            self._add_visible_admin_metadata(context, volume)
+
         limited_list = common.limited(volumes, req)
+        req.cache_resource(limited_list)
         res = [entity_maker(context, vol) for vol in limited_list]
         return {'volumes': res}
 
@@ -293,6 +357,7 @@ class VolumeController(wsgi.Controller):
         if not self.is_valid_body(body, 'volume'):
             raise exc.HTTPUnprocessableEntity()
 
+        LOG.debug('Create volume request body: %s', body)
         context = req.environ['cinder.context']
         volume = body['volume']
 
@@ -300,21 +365,17 @@ class VolumeController(wsgi.Controller):
 
         req_volume_type = volume.get('volume_type', None)
         if req_volume_type:
-            if not uuidutils.is_uuid_like(req_volume_type):
-                try:
+            try:
+                if not uuidutils.is_uuid_like(req_volume_type):
                     kwargs['volume_type'] = \
                         volume_types.get_volume_type_by_name(
                             context, req_volume_type)
-                except exception.VolumeTypeNotFound:
-                    explanation = 'Volume type not found.'
-                    raise exc.HTTPNotFound(explanation=explanation)
-            else:
-                try:
+                else:
                     kwargs['volume_type'] = volume_types.get_volume_type(
                         context, req_volume_type)
-                except exception.VolumeTypeNotFound:
-                    explanation = 'Volume type not found.'
-                    raise exc.HTTPNotFound(explanation=explanation)
+            except exception.VolumeTypeNotFound:
+                explanation = 'Volume type not found.'
+                raise exc.HTTPNotFound(explanation=explanation)
 
         kwargs['metadata'] = volume.get('metadata', None)
 
@@ -343,10 +404,8 @@ class VolumeController(wsgi.Controller):
         image_href = None
         image_uuid = None
         if self.ext_mgr.is_loaded('os-image-create'):
+            # NOTE(jdg): misleading name "imageRef" as it's an image-id
             image_href = volume.get('imageRef')
-            if snapshot_id and image_href:
-                msg = _("Snapshot and image cannot be specified together.")
-                raise exc.HTTPBadRequest(explanation=msg)
             if image_href:
                 image_uuid = self._image_uuid_from_href(image_href)
                 kwargs['image_id'] = image_uuid
@@ -362,15 +421,17 @@ class VolumeController(wsgi.Controller):
         # TODO(vish): Instance should be None at db layer instead of
         #             trying to lazy load, but for now we turn it into
         #             a dict to avoid an error.
-        retval = _translate_volume_detail_view(context,
-                                               dict(new_volume.iteritems()),
-                                               image_uuid)
+        new_volume = dict(new_volume.iteritems())
+
+        self._add_visible_admin_metadata(context, new_volume)
+
+        retval = _translate_volume_detail_view(context, new_volume, image_uuid)
 
         return {'volume': retval}
 
     def _get_volume_search_options(self):
         """Return volume search options allowed by non-admin."""
-        return ('display_name', 'status')
+        return ('display_name', 'status', 'metadata')
 
     @wsgi.serializers(xml=VolumeTemplate)
     def update(self, req, id, body):
@@ -404,6 +465,8 @@ class VolumeController(wsgi.Controller):
 
         volume.update(update_dict)
 
+        self._add_visible_admin_metadata(context, volume)
+
         return {'volume': _translate_volume_detail_view(context, volume)}
 
 
@@ -420,7 +483,8 @@ def remove_invalid_options(context, search_options, allowed_search_options):
     unknown_options = [opt for opt in search_options
                        if opt not in allowed_search_options]
     bad_options = ", ".join(unknown_options)
-    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
+    log_msg = _("Removing options '%(bad_options)s'"
+                " from query") % {'bad_options': bad_options}
     LOG.debug(log_msg)
     for opt in unknown_options:
         del search_options[opt]
