@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2011 OpenStack Foundation
 # Copyright 2011 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
@@ -28,17 +30,24 @@ from cinder.api.v2 import types
 from cinder.api.v2 import volume_metadata
 from cinder.api.v2 import volumes
 from cinder.api import versions
+from cinder.openstack.common import log as logging
+
+
+LOG = logging.getLogger(__name__)
 
 
 class APIRouter(cinder.api.openstack.APIRouter):
-    """Routes requests on the API to the appropriate controller and method."""
+    """
+    Routes requests on the OpenStack API to the appropriate controller
+    and method.
+    """
     ExtensionManager = extensions.ExtensionManager
 
     def _setup_routes(self, mapper, ext_mgr):
         self.resources['versions'] = versions.create_resource()
         mapper.connect("versions", "/",
                        controller=self.resources['versions'],
-                       action='index')
+                       action='show')
 
         mapper.redirect("", "/")
 
@@ -50,8 +59,7 @@ class APIRouter(cinder.api.openstack.APIRouter):
 
         self.resources['types'] = types.create_resource()
         mapper.resource("type", "types",
-                        controller=self.resources['types'],
-                        member={'action': 'POST'})
+                        controller=self.resources['types'])
 
         self.resources['snapshots'] = snapshots.create_resource(ext_mgr)
         mapper.resource("snapshot", "snapshots",
