@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -13,41 +15,47 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from six.moves import http_client
 import webob
-from webob import exc
 
+from cinder.api import common
 from cinder.api.openstack import wsgi
 from cinder import exception
-from cinder.i18n import _
 from cinder import volume
+from webob import exc
 
 
-class Controller(wsgi.Controller):
-    """The snapshot metadata API controller for the OpenStack API."""
+class Controller(object):
+    """The volume metadata API controller for the OpenStack API."""
 
     def __init__(self):
         self.volume_api = volume.API()
         super(Controller, self).__init__()
 
     def _get_metadata(self, context, snapshot_id):
-        return self._get_snapshot_and_metadata(context, snapshot_id)[1]
-
-    def _get_snapshot_and_metadata(self, context, snapshot_id):
-        # Not found exception will be handled at the wsgi level
-        snapshot = self.volume_api.get_snapshot(context, snapshot_id)
-        meta = self.volume_api.get_snapshot_metadata(context, snapshot)
-        return snapshot, meta
+        try:
+            snapshot = self.volume_api.get_snapshot(context, snapshot_id)
+            meta = self.volume_api.get_snapshot_metadata(context, snapshot)
+        except exception.SnapshotNotFound:
+            msg = _('snapshot does not exist')
+            raise exc.HTTPNotFound(explanation=msg)
+        return meta
 
+    @wsgi.serializers(xml=common.MetadataTemplate)
     def index(self, req, snapshot_id):
         """Returns the list of metadata for a given snapshot."""
         context = req.environ['cinder.context']
         return {'metadata': self._get_metadata(context, snapshot_id)}
 
+    @wsgi.serializers(xml=common.MetadataTemplate)
+    @wsgi.deserializers(xml=common.MetadataDeserializer)
     def create(self, req, snapshot_id, body):
-        self.assert_valid_body(body, 'metadata')
+        try:
+            metadata = body['metadata']
+        except (KeyError, TypeError):
+            msg = _("Malformed request body")
+            raise exc.HTTPBadRequest(explanation=msg)
+
         context = req.environ['cinder.context']
-        metadata = body['metadata']
 
         new_metadata = self._update_snapshot_metadata(context,
                                                       snapshot_id,
@@ -56,9 +64,14 @@ class Controller(wsgi.Controller):
 
         return {'metadata': new_metadata}
 
+    @wsgi.serializers(xml=common.MetaItemTemplate)
+    @wsgi.deserializers(xml=common.MetaItemDeserializer)
     def update(self, req, snapshot_id, id, body):
-        self.assert_valid_body(body, 'meta')
-        meta_item = body['meta']
+        try:
+            meta_item = body['meta']
+        except (TypeError, KeyError):
+            expl = _('Malformed request body')
+            raise exc.HTTPBadRequest(explanation=expl)
 
         if id not in meta_item:
             expl = _('Request body and URI mismatch')
@@ -76,11 +89,16 @@ class Controller(wsgi.Controller):
 
         return {'meta': meta_item}
 
+    @wsgi.serializers(xml=common.MetadataTemplate)
+    @wsgi.deserializers(xml=common.MetadataDeserializer)
     def update_all(self, req, snapshot_id, body):
-        self.assert_valid_body(body, 'metadata')
-        context = req.environ['cinder.context']
-        metadata = body['metadata']
+        try:
+            metadata = body['metadata']
+        except (TypeError, KeyError):
+            expl = _('Malformed request body')
+            raise exc.HTTPBadRequest(explanation=expl)
 
+        context = req.environ['cinder.context']
         new_metadata = self._update_snapshot_metadata(context,
                                                       snapshot_id,
                                                       metadata,
@@ -97,7 +115,10 @@ class Controller(wsgi.Controller):
                                                             snapshot,
                                                             metadata,
                                                             delete)
-        # Not found exception will be handled at the wsgi level
+        except exception.SnapshotNotFound:
+            msg = _('snapshot does not exist')
+            raise exc.HTTPNotFound(explanation=msg)
+
         except (ValueError, AttributeError):
             msg = _("Malformed request body")
             raise exc.HTTPBadRequest(explanation=msg)
@@ -108,6 +129,7 @@ class Controller(wsgi.Controller):
         except exception.InvalidVolumeMetadataSize as error:
             raise exc.HTTPRequestEntityTooLarge(explanation=error.msg)
 
+    @wsgi.serializers(xml=common.MetaItemTemplate)
     def show(self, req, snapshot_id, id):
         """Return a single metadata item."""
         context = req.environ['cinder.context']
@@ -116,22 +138,26 @@ class Controller(wsgi.Controller):
         try:
             return {'meta': {id: data[id]}}
         except KeyError:
-            raise exception.SnapshotMetadataNotFound(snapshot_id=snapshot_id,
-                                                     metadata_key=id)
+            msg = _("Metadata item was not found")
+            raise exc.HTTPNotFound(explanation=msg)
 
     def delete(self, req, snapshot_id, id):
         """Deletes an existing metadata."""
         context = req.environ['cinder.context']
 
-        snapshot, metadata = self._get_snapshot_and_metadata(context,
-                                                             snapshot_id)
+        metadata = self._get_metadata(context, snapshot_id)
 
         if id not in metadata:
-            raise exception.SnapshotMetadataNotFound(snapshot_id=snapshot_id,
-                                                     metadata_key=id)
+            msg = _("Metadata item was not found")
+            raise exc.HTTPNotFound(explanation=msg)
 
-        self.volume_api.delete_snapshot_metadata(context, snapshot, id)
-        return webob.Response(status_int=http_client.OK)
+        try:
+            snapshot = self.volume_api.get_snapshot(context, snapshot_id)
+            self.volume_api.delete_snapshot_metadata(context, snapshot, id)
+        except exception.SnapshotNotFound:
+            msg = _('snapshot does not exist')
+            raise exc.HTTPNotFound(explanation=msg)
+        return webob.Response(status_int=200)
 
 
 def create_resource():
