@@ -23,8 +23,8 @@ from cinder.api.openstack import wsgi
 from cinder.api.v2 import volumes
 from cinder.api import xmlutil
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
+from cinder.openstack.common import strutils
 from cinder import utils
 from cinder import volume
 
@@ -32,9 +32,6 @@ from cinder import volume
 LOG = logging.getLogger(__name__)
 
 
-FLAGS = flags.FLAGS
-
-
 def _translate_snapshot_detail_view(context, snapshot):
     """Maps keys for snapshots details view."""
 
@@ -111,7 +108,8 @@ class SnapshotsController(wsgi.Controller):
         try:
             vol = self.volume_api.get_snapshot(context, id)
         except exception.NotFound:
-            raise exc.HTTPNotFound()
+            msg = _("Snapshot could not be found")
+            raise exc.HTTPNotFound(explanation=msg)
 
         return {'snapshot': _translate_snapshot_detail_view(context, vol)}
 
@@ -125,7 +123,9 @@ class SnapshotsController(wsgi.Controller):
             snapshot = self.volume_api.get_snapshot(context, id)
             self.volume_api.delete_snapshot(context, snapshot)
         except exception.NotFound:
-            raise exc.HTTPNotFound()
+            msg = _("Snapshot could not be found")
+            raise exc.HTTPNotFound(explanation=msg)
+
         return webob.Response(status_int=202)
 
     @wsgi.serializers(xml=SnapshotsTemplate)
@@ -171,7 +171,9 @@ class SnapshotsController(wsgi.Controller):
         context = req.environ['cinder.context']
 
         if not self.is_valid_body(body, 'snapshot'):
-            raise exc.HTTPBadRequest()
+            msg = (_("Missing required element '%s' in request body") %
+                   'snapshot')
+            raise exc.HTTPBadRequest(explanation=msg)
 
         snapshot = body['snapshot']
         kwargs['metadata'] = snapshot.get('metadata', None)
@@ -191,7 +193,7 @@ class SnapshotsController(wsgi.Controller):
             msg = _("Invalid value '%s' for force. ") % force
             raise exception.InvalidParameterValue(err=msg)
 
-        if utils.bool_from_str(force):
+        if strutils.bool_from_string(force):
             new_snapshot = self.volume_api.create_snapshot_force(
                 context,
                 volume,
@@ -216,10 +218,13 @@ class SnapshotsController(wsgi.Controller):
         context = req.environ['cinder.context']
 
         if not body:
-            raise exc.HTTPBadRequest()
+            msg = _("Missing request body")
+            raise exc.HTTPBadRequest(explanation=msg)
 
         if 'snapshot' not in body:
-            raise exc.HTTPBadRequest()
+            msg = (_("Missing required element '%s' in request body") %
+                   'snapshot')
+            raise exc.HTTPBadRequest(explanation=msg)
 
         snapshot = body['snapshot']
         update_dict = {}
@@ -250,7 +255,8 @@ class SnapshotsController(wsgi.Controller):
             snapshot = self.volume_api.get_snapshot(context, id)
             self.volume_api.update_snapshot(context, snapshot, update_dict)
         except exception.NotFound:
-            raise exc.HTTPNotFound()
+            msg = _("Snapshot could not be found")
+            raise exc.HTTPNotFound(explanation=msg)
 
         snapshot.update(update_dict)
 
