@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -12,20 +14,16 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-from http import HTTPStatus
 
 import webob
 
 from cinder.api import common
 from cinder.api.openstack import wsgi
-from cinder.api.schemas import volume_metadata as metadata
-from cinder.api import validation
 from cinder import exception
-from cinder.i18n import _
 from cinder import volume
 
 
-class Controller(wsgi.Controller):
+class Controller(object):
     """The volume metadata API controller for the OpenStack API."""
 
     def __init__(self):
@@ -33,34 +31,46 @@ class Controller(wsgi.Controller):
         super(Controller, self).__init__()
 
     def _get_metadata(self, context, volume_id):
-        # The metadata is at the second position of the tuple returned
-        # from _get_volume_and_metadata
-        return self._get_volume_and_metadata(context, volume_id)[1]
-
-    def _get_volume_and_metadata(self, context, volume_id):
-        # Not found exception will be handled at the wsgi level
-        volume = self.volume_api.get(context, volume_id)
-        meta = self.volume_api.get_volume_metadata(context, volume)
-        return (volume, meta)
+        try:
+            volume = self.volume_api.get(context, volume_id)
+            meta = self.volume_api.get_volume_metadata(context, volume)
+        except exception.VolumeNotFound:
+            msg = _('volume does not exist')
+            raise webob.exc.HTTPNotFound(explanation=msg)
+        return meta
 
+    @wsgi.serializers(xml=common.MetadataTemplate)
     def index(self, req, volume_id):
         """Returns the list of metadata for a given volume."""
         context = req.environ['cinder.context']
         return {'metadata': self._get_metadata(context, volume_id)}
 
-    @validation.schema(metadata.create)
+    @wsgi.serializers(xml=common.MetadataTemplate)
+    @wsgi.deserializers(xml=common.MetadataDeserializer)
     def create(self, req, volume_id, body):
+        try:
+            metadata = body['metadata']
+        except (KeyError, TypeError):
+            msg = _("Malformed request body")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         context = req.environ['cinder.context']
-        metadata = body['metadata']
+
+        new_metadata = self._update_volume_metadata(context,
+                                                    volume_id,
+                                                    metadata,
+                                                    delete=False)
 
-        new_metadata = self._update_volume_metadata(context, volume_id,
-                                                    metadata, delete=False,
-                                                    use_create=True)
         return {'metadata': new_metadata}
 
-    @validation.schema(metadata.update)
+    @wsgi.serializers(xml=common.MetaItemTemplate)
+    @wsgi.deserializers(xml=common.MetaItemDeserializer)
     def update(self, req, volume_id, id, body):
-        meta_item = body['meta']
+        try:
+            meta_item = body['meta']
+        except (TypeError, KeyError):
+            expl = _('Malformed request body')
+            raise webob.exc.HTTPBadRequest(explanation=expl)
 
         if id not in meta_item:
             expl = _('Request body and URI mismatch')
@@ -78,11 +88,16 @@ class Controller(wsgi.Controller):
 
         return {'meta': meta_item}
 
-    @validation.schema(metadata.create)
+    @wsgi.serializers(xml=common.MetadataTemplate)
+    @wsgi.deserializers(xml=common.MetadataDeserializer)
     def update_all(self, req, volume_id, body):
-        metadata = body['metadata']
-        context = req.environ['cinder.context']
+        try:
+            metadata = body['metadata']
+        except (TypeError, KeyError):
+            expl = _('Malformed request body')
+            raise webob.exc.HTTPBadRequest(explanation=expl)
 
+        context = req.environ['cinder.context']
         new_metadata = self._update_volume_metadata(context,
                                                     volume_id,
                                                     metadata,
@@ -90,18 +105,19 @@ class Controller(wsgi.Controller):
 
         return {'metadata': new_metadata}
 
-    def _update_volume_metadata(self, context, volume_id, metadata,
-                                delete=False, use_create=False):
+    def _update_volume_metadata(self, context,
+                                volume_id, metadata,
+                                delete=False):
         try:
             volume = self.volume_api.get(context, volume_id)
-            if use_create:
-                return self.volume_api.create_volume_metadata(context, volume,
-                                                              metadata)
-            else:
-                return self.volume_api.update_volume_metadata(
-                    context, volume, metadata, delete,
-                    meta_type=common.METADATA_TYPES.user)
-        # Not found exception will be handled at the wsgi level
+            return self.volume_api.update_volume_metadata(context,
+                                                          volume,
+                                                          metadata,
+                                                          delete)
+        except exception.VolumeNotFound:
+            msg = _('volume does not exist')
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
         except (ValueError, AttributeError):
             msg = _("Malformed request body")
             raise webob.exc.HTTPBadRequest(explanation=msg)
@@ -112,6 +128,7 @@ class Controller(wsgi.Controller):
         except exception.InvalidVolumeMetadataSize as error:
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=error.msg)
 
+    @wsgi.serializers(xml=common.MetaItemTemplate)
     def show(self, req, volume_id, id):
         """Return a single metadata item."""
         context = req.environ['cinder.context']
@@ -120,26 +137,26 @@ class Controller(wsgi.Controller):
         try:
             return {'meta': {id: data[id]}}
         except KeyError:
-            raise exception.VolumeMetadataNotFound(volume_id=volume_id,
-                                                   metadata_key=id)
+            msg = _("Metadata item was not found")
+            raise webob.exc.HTTPNotFound(explanation=msg)
 
     def delete(self, req, volume_id, id):
         """Deletes an existing metadata."""
         context = req.environ['cinder.context']
 
-        volume, metadata = self._get_volume_and_metadata(context, volume_id)
+        metadata = self._get_metadata(context, volume_id)
 
         if id not in metadata:
-            raise exception.VolumeMetadataNotFound(volume_id=volume_id,
-                                                   metadata_key=id)
+            msg = _("Metadata item was not found")
+            raise webob.exc.HTTPNotFound(explanation=msg)
 
-        # Not found exception will be handled at the wsgi level
-        self.volume_api.delete_volume_metadata(
-            context,
-            volume,
-            id,
-            meta_type=common.METADATA_TYPES.user)
-        return webob.Response(status_int=HTTPStatus.OK)
+        try:
+            volume = self.volume_api.get(context, volume_id)
+            self.volume_api.delete_volume_metadata(context, volume, id)
+        except exception.VolumeNotFound:
+            msg = _('volume does not exist')
+            raise webob.exc.HTTPNotFound(explanation=msg)
+        return webob.Response(status_int=200)
 
 
 def create_resource():
