@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -15,6 +17,8 @@
 
 import datetime
 
+from cinder.openstack.common import timeutils
+
 
 class ViewBuilder(object):
     """OpenStack API base limits view builder."""
@@ -40,14 +44,19 @@ class ViewBuilder(object):
 
         """
         limit_names = {
+            "ram": ["maxTotalRAMSize"],
+            "instances": ["maxTotalInstances"],
+            "cores": ["maxTotalCores"],
             "gigabytes": ["maxTotalVolumeGigabytes"],
-            "backup_gigabytes": ["maxTotalBackupGigabytes"],
             "volumes": ["maxTotalVolumes"],
-            "snapshots": ["maxTotalSnapshots"],
-            "backups": ["maxTotalBackups"],
+            "key_pairs": ["maxTotalKeypairs"],
+            "floating_ips": ["maxTotalFloatingIps"],
+            "metadata_items": ["maxServerMeta", "maxImageMeta"],
+            "injected_files": ["maxPersonality"],
+            "injected_file_content_bytes": ["maxPersonalitySize"],
         }
         limits = {}
-        for name, value in absolute_limits.items():
+        for name, value in absolute_limits.iteritems():
             if name in limit_names and value is not None:
                 for name in limit_names[name]:
                     limits[name] = value
@@ -87,5 +96,5 @@ class ViewBuilder(object):
             "value": rate_limit["value"],
             "remaining": int(rate_limit["remaining"]),
             "unit": rate_limit["unit"],
-            "next-available": next_avail.isoformat(),
+            "next-available": timeutils.isotime(at=next_avail),
         }
