@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # Copyright 2010-2011 OpenStack Foundation
@@ -17,50 +19,19 @@
 #    under the License.
 
 """Implementation of paginate query."""
-import datetime
 
-from oslo_log import log as logging
 import sqlalchemy
-import sqlalchemy.sql as sa_sql
-from sqlalchemy.sql import type_api
 
-from cinder.db import api
 from cinder import exception
-from cinder.i18n import _
+from cinder.openstack.common import log as logging
 
 
 LOG = logging.getLogger(__name__)
 
-_TYPE_SCHEMA = {
-    'datetime': datetime.datetime(1900, 1, 1),
-    'big_integer': 0,
-    'integer': 0,
-    'string': '',
-    'boolean': False,
-}
 
-
-def _get_default_column_value(model, column_name):
-    """Return the default value of the columns from DB table.
-
-    In postgreDB case, if no right default values are being set, an
-    psycopg2.DataError will be thrown.
-    """
-    attr = getattr(model, column_name)
-    # Return the default value directly if the model contains. Otherwise return
-    # a default value which is not None.
-    if attr.default and isinstance(attr.default, type_api.TypeEngine):
-        return attr.default.arg
-
-    attr_type = attr.type
-    return _TYPE_SCHEMA[attr_type.__visit_name__]
-
-
-# TODO(wangxiyuan): Use oslo_db.sqlalchemy.utils.paginate_query once it is
-# stable and afforded by the minimum version in requirement.txt.
 # copied from glance/db/sqlalchemy/api.py
 def paginate_query(query, model, limit, sort_keys, marker=None,
-                   sort_dir=None, sort_dirs=None, offset=None):
+                   sort_dir=None, sort_dirs=None):
     """Returns a query with sorting / pagination criteria added.
 
     Pagination works by requiring a unique sort_key, specified by sort_keys.
@@ -86,8 +57,6 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
                     results after this value.
     :param sort_dir: direction in which results should be sorted (asc, desc)
     :param sort_dirs: per-column array of sort_dirs, corresponding to sort_keys
-    :param offset: the number of items to skip from the marker or from the
-                    first element.
 
     :rtype: sqlalchemy.orm.query.Query
     :return: The query with sorting/pagination added.
@@ -96,10 +65,9 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     if 'id' not in sort_keys:
         # TODO(justinsb): If this ever gives a false-positive, check
         # the actual primary key, rather than assuming its id
-        LOG.warning('Id not in sort_keys; is sort_keys unique?')
+        LOG.warn(_('Id not in sort_keys; is sort_keys unique?'))
 
-    if sort_dir and sort_dirs:
-        raise AssertionError('Both sort_dir and sort_dirs specified.')
+    assert(not (sort_dir and sort_dirs))
 
     # Default the sort direction to ascending
     if sort_dirs is None and sort_dir is None:
@@ -109,9 +77,7 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     if sort_dirs is None:
         sort_dirs = [sort_dir for _sort_key in sort_keys]
 
-    if len(sort_dirs) != len(sort_keys):
-        raise AssertionError(
-            'sort_dirs length is not equal to sort_keys length.')
+    assert(len(sort_dirs) == len(sort_keys))
 
     # Add sorting
     for current_sort_key, current_sort_dir in zip(sort_keys, sort_dirs):
@@ -124,8 +90,6 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
             sort_key_attr = getattr(model, current_sort_key)
         except AttributeError:
             raise exception.InvalidInput(reason='Invalid sort key')
-        if not api.is_orm_value(sort_key_attr):
-            raise exception.InvalidInput(reason='Invalid sort key')
         query = query.order_by(sort_dir_func(sort_key_attr))
 
     # Add pagination
@@ -133,35 +97,21 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
         marker_values = []
         for sort_key in sort_keys:
             v = getattr(marker, sort_key)
-            if v is None:
-                v = _get_default_column_value(model, sort_key)
             marker_values.append(v)
 
         # Build up an array of sort criteria as in the docstring
         criteria_list = []
-        for i in range(0, len(sort_keys)):
+        for i in xrange(0, len(sort_keys)):
             crit_attrs = []
-            for j in range(0, i):
+            for j in xrange(0, i):
                 model_attr = getattr(model, sort_keys[j])
-                default = _get_default_column_value(model, sort_keys[j])
-                attr = sa_sql.expression.case(
-                    *[(model_attr.isnot(None), model_attr)],
-                    else_=default,
-                )
-                crit_attrs.append((attr == marker_values[j]))
+                crit_attrs.append((model_attr == marker_values[j]))
 
             model_attr = getattr(model, sort_keys[i])
-            default = _get_default_column_value(model, sort_keys[i])
-            attr = sa_sql.expression.case(
-                *[(model_attr.isnot(None), model_attr)],
-                else_=default,
-            )
-            if isinstance(model_attr.type, sqlalchemy.Boolean):
-                marker_values[i] = int(marker_values[i])
             if sort_dirs[i] == 'desc':
-                crit_attrs.append((attr < marker_values[i]))
+                crit_attrs.append((model_attr < marker_values[i]))
             elif sort_dirs[i] == 'asc':
-                crit_attrs.append((attr > marker_values[i]))
+                crit_attrs.append((model_attr > marker_values[i]))
             else:
                 raise ValueError(_("Unknown sort direction, "
                                    "must be 'desc' or 'asc'"))
@@ -175,7 +125,4 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     if limit is not None:
         query = query.limit(limit)
 
-    if offset:
-        query = query.offset(offset)
-
     return query
