@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 OpenStack LLC.
+# Copyright 2012 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -35,18 +35,10 @@ def upgrade(migrate_engine):
                                                     name=None)),
                           Column('id', Integer(), primary_key=True),
                           Column('class_name',
-                                 String(length=255,
-                                        convert_unicode=True,
-                                        assert_unicode=None,
-                                        unicode_error=None,
-                                        _warn_on_bytestring=False),
+                                 String(length=255),
                                  index=True),
                           Column('resource',
-                                 String(length=255,
-                                        convert_unicode=True,
-                                        assert_unicode=None,
-                                        unicode_error=None,
-                                        _warn_on_bytestring=False)),
+                                 String(length=255)),
                           Column('hard_limit', Integer(), nullable=True),
                           mysql_engine='InnoDB',
                           mysql_charset='utf8',
@@ -66,14 +58,10 @@ def upgrade(migrate_engine):
                                                    name=None)),
                          Column('id', Integer(), primary_key=True),
                          Column('project_id',
-                                String(length=255, convert_unicode=True,
-                                       assert_unicode=None, unicode_error=None,
-                                       _warn_on_bytestring=False),
+                                String(length=255),
                                 index=True),
                          Column('resource',
-                                String(length=255, convert_unicode=True,
-                                       assert_unicode=None, unicode_error=None,
-                                       _warn_on_bytestring=False)),
+                                String(length=255)),
                          Column('in_use', Integer(), nullable=False),
                          Column('reserved', Integer(), nullable=False),
                          Column('until_refresh', Integer(), nullable=True),
@@ -95,25 +83,17 @@ def upgrade(migrate_engine):
                                                    name=None)),
                          Column('id', Integer(), primary_key=True),
                          Column('uuid',
-                                String(length=36,
-                                       convert_unicode=True,
-                                       assert_unicode=None,
-                                       unicode_error=None,
-                                       _warn_on_bytestring=False),
+                                String(length=36),
                                 nullable=False),
                          Column('usage_id',
                                 Integer(),
                                 ForeignKey('quota_usages.id'),
                                 nullable=False),
                          Column('project_id',
-                                String(length=255, convert_unicode=True,
-                                       assert_unicode=None, unicode_error=None,
-                                       _warn_on_bytestring=False),
+                                String(length=255),
                                 index=True),
                          Column('resource',
-                                String(length=255, convert_unicode=True,
-                                       assert_unicode=None, unicode_error=None,
-                                       _warn_on_bytestring=False)),
+                                String(length=255)),
                          Column('delta', Integer(), nullable=False),
                          Column('expire', DateTime(timezone=False)),
                          mysql_engine='InnoDB',
