@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2012 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,6 +17,10 @@
 from sqlalchemy import Column, DateTime, Text, Boolean
 from sqlalchemy import MetaData, Integer, String, Table, ForeignKey
 
+from cinder.openstack.common import log as logging
+
+LOG = logging.getLogger(__name__)
+
 
 def upgrade(migrate_engine):
     meta = MetaData()
@@ -23,14 +29,16 @@ def upgrade(migrate_engine):
     # Just for the ForeignKey and column creation to succeed, these are not the
     # actual definitions of tables .
     #
-    Table('volumes',
-          meta,
-          Column('id', Integer(), primary_key=True, nullable=False),
-          mysql_engine='InnoDB')
-    Table('snapshots',
-          meta,
-          Column('id', Integer(), primary_key=True, nullable=False),
-          mysql_engine='InnoDB')
+    volumes = Table('volumes',
+                    meta,
+                    Column('id', Integer(),
+                           primary_key=True, nullable=False),
+                    mysql_engine='InnoDB')
+    snapshots = Table('snapshots',
+                      meta,
+                      Column('id', Integer(),
+                             primary_key=True, nullable=False),
+                      mysql_engine='InnoDB')
     # Create new table
     volume_glance_metadata = Table(
         'volume_glance_metadata',
@@ -51,5 +59,20 @@ def upgrade(migrate_engine):
     try:
         volume_glance_metadata.create()
     except Exception:
+        LOG.exception(_("Exception while creating table "
+                        "'volume_glance_metadata'"))
         meta.drop_all(tables=[volume_glance_metadata])
         raise
+
+
+def downgrade(migrate_engine):
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    volume_glance_metadata = Table('volume_glance_metadata',
+                                   meta, autoload=True)
+    try:
+        volume_glance_metadata.drop()
+    except Exception:
+        LOG.error(_("volume_glance_metadata table not dropped"))
+        raise
