@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,12 +15,16 @@
 from sqlalchemy import Boolean, Column, DateTime
 from sqlalchemy import MetaData, String, Table, ForeignKey
 
+from cinder.openstack.common import log as logging
+
+LOG = logging.getLogger(__name__)
+
 
 def upgrade(migrate_engine):
     meta = MetaData()
     meta.bind = migrate_engine
 
-    Table('volumes', meta, autoload=True)
+    volumes = Table('volumes', meta, autoload=True)
 
     # New table
     transfers = Table(
@@ -38,4 +44,20 @@ def upgrade(migrate_engine):
         mysql_charset='utf8'
     )
 
-    transfers.create()
+    try:
+        transfers.create()
+    except Exception:
+        LOG.error(_("Table |%s| not created!"), repr(transfers))
+        raise
+
+
+def downgrade(migrate_engine):
+    meta = MetaData()
+    meta.bind = migrate_engine
+    transfers = Table('transfers',
+                      meta,
+                      autoload=True)
+    try:
+        transfers.drop()
+    except Exception:
+        LOG.error(_("transfers table not dropped"))
