@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -31,3 +33,14 @@ def upgrade(migrate_engine):
         volumes.update().\
             where(volumes.c.id == item['volume_id']).\
             values(bootable=True).execute()
+
+
+def downgrade(migrate_engine):
+    """Remove bootable column to volumes."""
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    volumes = Table('volumes', meta, autoload=True)
+    bootable = volumes.columns.bootable
+    #bootable = Column('bootable', Boolean)
+    volumes.drop_column(bootable)
