@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -23,3 +25,13 @@ def upgrade(migrate_engine):
     provider_geometry = Column('provider_geometry', String(255))
     volumes.create_column(provider_geometry)
     volumes.update().values(provider_geometry=None).execute()
+
+
+def downgrade(migrate_engine):
+    """Remove provider_geometry column from volumes."""
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    volumes = Table('volumes', meta, autoload=True)
+    provider_geometry = Column('provider_geometry', String(255))
+    volumes.drop_column(provider_geometry)
