@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -23,3 +25,13 @@ def upgrade(migrate_engine):
     _name_id = Column('_name_id', String(36))
     volumes.create_column(_name_id)
     volumes.update().values(_name_id=None).execute()
+
+
+def downgrade(migrate_engine):
+    """Remove _name_id column from volumes."""
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    volumes = Table('volumes', meta, autoload=True)
+    _name_id = volumes.columns._name_id
+    volumes.drop_column(_name_id)
