@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright (C) 2013 eBay Inc.
 # Copyright (C) 2013 OpenStack Foundation
 # All Rights Reserved.
@@ -15,7 +17,11 @@
 #    under the License.
 
 from sqlalchemy import Boolean, Column, DateTime
-from sqlalchemy import ForeignKey, MetaData, String, Table
+from sqlalchemy import ForeignKey, MetaData, Integer, String, Table
+
+from cinder.openstack.common import log as logging
+
+LOG = logging.getLogger(__name__)
 
 
 def upgrade(migrate_engine):
@@ -38,11 +44,42 @@ def upgrade(migrate_engine):
         mysql_charset='utf8'
     )
 
-    quality_of_service_specs.create()
+    try:
+        quality_of_service_specs.create()
+    except Exception:
+        LOG.error(_("Table quality_of_service_specs not created!"))
+        raise
 
     volume_types = Table('volume_types', meta, autoload=True)
     qos_specs_id = Column('qos_specs_id', String(36),
                           ForeignKey('quality_of_service_specs.id'))
 
-    volume_types.create_column(qos_specs_id)
-    volume_types.update().values(qos_specs_id=None).execute()
+    try:
+        volume_types.create_column(qos_specs_id)
+        volume_types.update().values(qos_specs_id=None).execute()
+    except Exception:
+        LOG.error(_("Added qos_specs_id column to volume type table failed."))
+        raise
+
+
+def downgrade(migrate_engine):
+    """Remove volume_type_rate_limit table."""
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    qos_specs = Table('quality_of_service_specs', meta, autoload=True)
+
+    try:
+        qos_specs.drop()
+
+    except Exception:
+        LOG.error(_("Dropping quality_of_service_specs table failed."))
+        raise
+
+    volume_types = Table('volume_types', meta, autoload=True)
+    qos_specs_id = Column('qos_specs_id', String(36))
+    try:
+        volume_types.drop_column(qos_specs_id)
+    except Exception:
+        LOG.error(_("Dropping qos_specs_id column failed."))
+        raise
