@@ -24,11 +24,14 @@ SHOULD include dedicated exception logging.
 
 """
 
+import sys
+
 from oslo.config import cfg
 import webob.exc
 
-from cinder import flags
 from cinder.openstack.common import log as logging
+from cinder.openstack.common import processutils
+
 
 LOG = logging.getLogger(__name__)
 
@@ -38,8 +41,8 @@ exc_log_opts = [
                 help='make exception message format errors fatal'),
 ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(exc_log_opts)
+CONF = cfg.CONF
+CONF.register_opts(exc_log_opts)
 
 
 class ConvertedException(webob.exc.WSGIHTTPException):
@@ -50,49 +53,10 @@ class ConvertedException(webob.exc.WSGIHTTPException):
         super(ConvertedException, self).__init__()
 
 
-class ProcessExecutionError(IOError):
-    def __init__(self, stdout=None, stderr=None, exit_code=None, cmd=None,
-                 description=None):
-        self.exit_code = exit_code
-        self.stderr = stderr
-        self.stdout = stdout
-        self.cmd = cmd
-        self.description = description
-
-        if description is None:
-            description = _('Unexpected error while running command.')
-        if exit_code is None:
-            exit_code = '-'
-        message = _('%(description)s\nCommand: %(cmd)s\n'
-                    'Exit code: %(exit_code)s\nStdout: %(stdout)r\n'
-                    'Stderr: %(stderr)r') % locals()
-        IOError.__init__(self, message)
-
-
 class Error(Exception):
     pass
 
 
-class DBError(Error):
-    """Wraps an implementation specific exception."""
-    def __init__(self, inner_exception=None):
-        self.inner_exception = inner_exception
-        super(DBError, self).__init__(str(inner_exception))
-
-
-def wrap_db_error(f):
-    def _wrap(*args, **kwargs):
-        try:
-            return f(*args, **kwargs)
-        except UnicodeEncodeError:
-            raise InvalidUnicodeParameter()
-        except Exception, e:
-            LOG.exception(_('DB exception wrapped.'))
-            raise DBError(e)
-    _wrap.func_name = f.func_name
-    return _wrap
-
-
 class CinderException(Exception):
     """Base Cinder Exception
 
@@ -119,23 +83,30 @@ class CinderException(Exception):
             try:
                 message = self.message % kwargs
 
-            except Exception as e:
+            except Exception:
+                exc_info = sys.exc_info()
                 # kwargs doesn't match a variable in the message
                 # log the issue and the kwargs
                 LOG.exception(_('Exception in string format operation'))
                 for name, value in kwargs.iteritems():
                     LOG.error("%s: %s" % (name, value))
-                if FLAGS.fatal_exception_format_errors:
-                    raise e
-                else:
-                    # at least get the core message out if something happened
-                    message = self.message
+                if CONF.fatal_exception_format_errors:
+                    raise exc_info[0], exc_info[1], exc_info[2]
+                # at least get the core message out if something happened
+                message = self.message
 
+        # NOTE(luisg): We put the actual message in 'msg' so that we can access
+        # it, because if we try to access the message via 'message' it will be
+        # overshadowed by the class' message attribute
+        self.msg = message
         super(CinderException, self).__init__(message)
 
+    def __unicode__(self):
+        return unicode(self.msg)
+
 
 class GlanceConnectionFailed(CinderException):
-    message = _("Connection to glance failed") + ": %(reason)s"
+    message = _("Connection to glance failed: %(reason)s")
 
 
 class NotAuthorized(CinderException):
@@ -155,13 +126,22 @@ class ImageNotAuthorized(CinderException):
     message = _("Not authorized for image %(image_id)s.")
 
 
+class DriverNotInitialized(CinderException):
+    message = _("Volume driver not ready.")
+
+
 class Invalid(CinderException):
     message = _("Unacceptable parameters.")
     code = 400
 
 
 class InvalidSnapshot(Invalid):
-    message = _("Invalid snapshot") + ": %(reason)s"
+    message = _("Invalid snapshot: %(reason)s")
+
+
+class InvalidVolumeAttachMode(Invalid):
+    message = _("Invalid attaching mode '%(mode)s' for "
+                "volume %(volume_id)s.")
 
 
 class VolumeAttached(Invalid):
@@ -172,37 +152,28 @@ class SfJsonEncodeFailure(CinderException):
     message = _("Failed to load data into json format")
 
 
-class InvalidRequest(Invalid):
-    message = _("The request is invalid.")
-
-
 class InvalidResults(Invalid):
     message = _("The results are invalid.")
 
 
 class InvalidInput(Invalid):
-    message = _("Invalid input received") + ": %(reason)s"
+    message = _("Invalid input received: %(reason)s")
 
 
 class InvalidVolumeType(Invalid):
-    message = _("Invalid volume type") + ": %(reason)s"
+    message = _("Invalid volume type: %(reason)s")
 
 
 class InvalidVolume(Invalid):
-    message = _("Invalid volume") + ": %(reason)s"
-
-
-class InvalidPortRange(Invalid):
-    message = _("Invalid port range %(from_port)s:%(to_port)s. %(msg)s")
+    message = _("Invalid volume: %(reason)s")
 
 
 class InvalidContentType(Invalid):
     message = _("Invalid content type %(content_type)s.")
 
 
-class InvalidUnicodeParameter(Invalid):
-    message = _("Invalid Parameter: "
-                "Unicode is not supported by the current database.")
+class InvalidHost(Invalid):
+    message = _("Invalid host: %(reason)s")
 
 
 # Cannot be templated as the error syntax varies.
@@ -211,6 +182,15 @@ class InvalidParameterValue(Invalid):
     message = _("%(err)s")
 
 
+class InvalidAuthKey(Invalid):
+    message = _("Invalid auth key: %(reason)s")
+
+
+class InvalidConfigurationValue(Invalid):
+    message = _('Value "%(value)s" is not valid for '
+                'configuration option "%(option)s"')
+
+
 class ServiceUnavailable(Invalid):
     message = _("Service is unavailable at this time.")
 
@@ -219,8 +199,12 @@ class ImageUnacceptable(Invalid):
     message = _("Image %(image_id)s is unacceptable: %(reason)s")
 
 
+class DeviceUnavailable(Invalid):
+    message = _("The device in the path %(path)s is unavailable: %(reason)s")
+
+
 class InvalidUUID(Invalid):
-    message = _("Expected a uuid but received %(uuid).")
+    message = _("Expected a uuid but received %(uuid)s.")
 
 
 class NotFound(CinderException):
@@ -229,10 +213,6 @@ class NotFound(CinderException):
     safe = True
 
 
-class PersistentVolumeFileNotFound(NotFound):
-    message = _("Volume %(volume_id)s persistence file could not be found.")
-
-
 class VolumeNotFound(NotFound):
     message = _("Volume %(volume_id)s could not be found.")
 
@@ -242,21 +222,22 @@ class SfAccountNotFound(NotFound):
                 "Solidfire device")
 
 
-class VolumeNotFoundForInstance(VolumeNotFound):
-    message = _("Volume not found for instance %(instance_id)s.")
-
-
 class VolumeMetadataNotFound(NotFound):
     message = _("Volume %(volume_id)s has no metadata with "
                 "key %(metadata_key)s.")
 
 
+class VolumeAdminMetadataNotFound(NotFound):
+    message = _("Volume %(volume_id)s has no administration metadata with "
+                "key %(metadata_key)s.")
+
+
 class InvalidVolumeMetadata(Invalid):
-    message = _("Invalid metadata") + ": %(reason)s"
+    message = _("Invalid metadata: %(reason)s")
 
 
 class InvalidVolumeMetadataSize(Invalid):
-    message = _("Invalid metadata size") + ": %(reason)s"
+    message = _("Invalid metadata size: %(reason)s")
 
 
 class SnapshotMetadataNotFound(NotFound):
@@ -264,14 +245,6 @@ class SnapshotMetadataNotFound(NotFound):
                 "key %(metadata_key)s.")
 
 
-class InvalidSnapshotMetadata(Invalid):
-    message = _("Invalid metadata") + ": %(reason)s"
-
-
-class InvalidSnapshotMetadataSize(Invalid):
-    message = _("Invalid metadata size") + ": %(reason)s"
-
-
 class VolumeTypeNotFound(NotFound):
     message = _("Volume type %(volume_type_id)s could not be found.")
 
@@ -286,6 +259,11 @@ class VolumeTypeExtraSpecsNotFound(NotFound):
                 "key %(extra_specs_key)s.")
 
 
+class VolumeTypeInUse(CinderException):
+    message = _("Volume Type %(volume_type_id)s deletion is not allowed with "
+                "volumes present with the type.")
+
+
 class SnapshotNotFound(NotFound):
     message = _("Snapshot %(snapshot_id)s could not be found.")
 
@@ -303,22 +281,6 @@ class ISCSITargetNotFoundForVolume(NotFound):
     message = _("No target id found for volume %(volume_id)s.")
 
 
-class ISCSITargetCreateFailed(CinderException):
-    message = _("Failed to create iscsi target for volume %(volume_id)s.")
-
-
-class ISCSITargetAttachFailed(CinderException):
-    message = _("Failed to attach iSCSI target for volume %(volume_id)s.")
-
-
-class ISCSITargetRemoveFailed(CinderException):
-    message = _("Failed to remove iscsi target for volume %(volume_id)s.")
-
-
-class DiskNotFound(NotFound):
-    message = _("No disk at %(location)s")
-
-
 class InvalidImageRef(Invalid):
     message = _("Invalid image href %(image_href)s.")
 
@@ -388,38 +350,21 @@ class MigrationNotFound(NotFound):
     message = _("Migration %(migration_id)s could not be found.")
 
 
-class MigrationNotFoundByStatus(MigrationNotFound):
-    message = _("Migration not found for instance %(instance_id)s "
-                "with status %(status)s.")
-
-
 class FileNotFound(NotFound):
     message = _("File %(file_path)s could not be found.")
 
 
-class ClassNotFound(NotFound):
-    message = _("Class %(class_name)s could not be found: %(exception)s")
-
-
-class NotAllowed(CinderException):
-    message = _("Action not allowed.")
-
-
 #TODO(bcwaldon): EOL this exception!
 class Duplicate(CinderException):
     pass
 
 
-class KeyPairExists(Duplicate):
-    message = _("Key pair %(key_name)s already exists.")
-
-
 class VolumeTypeExists(Duplicate):
     message = _("Volume Type %(id)s already exists.")
 
 
-class MigrationError(CinderException):
-    message = _("Migration error") + ": %(reason)s"
+class VolumeTypeEncryptionExists(Invalid):
+    message = _("Volume type encryption for type %(type_id)s already exists.")
 
 
 class MalformedRequestBody(CinderException):
@@ -430,6 +375,10 @@ class ConfigNotFound(NotFound):
     message = _("Could not find config at %(path)s")
 
 
+class ParameterNotFound(NotFound):
+    message = _("Could not find parameter %(param)s")
+
+
 class PasteAppNotFound(NotFound):
     message = _("Could not load paste app '%(name)s' from %(path)s")
 
@@ -443,7 +392,7 @@ class WillNotSchedule(CinderException):
 
 
 class QuotaError(CinderException):
-    message = _("Quota exceeded") + ": code=%(code)s"
+    message = _("Quota exceeded: code=%(code)s")
     code = 413
     headers = {'Retry-After': 0}
     safe = True
@@ -454,10 +403,6 @@ class VolumeSizeExceedsAvailableQuota(QuotaError):
                 "allowed Gigabytes quota")
 
 
-class VolumeSizeExceedsQuota(QuotaError):
-    message = _("Maximum volume/snapshot size exceeded")
-
-
 class VolumeLimitExceeded(QuotaError):
     message = _("Maximum number of volumes allowed (%(allowed)d) exceeded")
 
@@ -470,8 +415,8 @@ class DuplicateSfVolumeNames(Duplicate):
     message = _("Detected more than one volume with name %(vol_name)s")
 
 
-class Duplicate3PARHost(CinderException):
-    message = _("3PAR Host already exists: %(err)s.  %(info)s")
+class Invalid3PARDomain(CinderException):
+    message = _("Invalid 3PAR Domain: %(err)s")
 
 
 class VolumeTypeCreateFailed(CinderException):
@@ -565,17 +510,134 @@ class GlanceMetadataNotFound(NotFound):
     message = _("Glance metadata for volume/snapshot %(id)s cannot be found.")
 
 
+class ExportFailure(Invalid):
+    message = _("Failed to export for volume: %(reason)s")
+
+
+class MetadataCreateFailure(Invalid):
+    message = _("Failed to create metadata for volume: %(reason)s")
+
+
+class MetadataUpdateFailure(Invalid):
+    message = _("Failed to update metadata for volume: %(reason)s")
+
+
+class MetadataCopyFailure(Invalid):
+    message = _("Failed to copy metadata to volume: %(reason)s")
+
+
 class ImageCopyFailure(Invalid):
     message = _("Failed to copy image to volume: %(reason)s")
 
 
+class BackupInvalidCephArgs(Invalid):
+    message = _("Invalid Ceph args provided for backup rbd operation")
+
+
+class BackupOperationError(Invalid):
+    message = _("An error has occurred during backup operation")
+
+
+class BackupRBDOperationFailed(Invalid):
+    message = _("Backup RBD operation failed")
+
+
 class BackupNotFound(NotFound):
     message = _("Backup %(backup_id)s could not be found.")
 
 
+class BackupFailedToGetVolumeBackend(NotFound):
+    message = _("Failed to identify volume backend.")
+
+
 class InvalidBackup(Invalid):
     message = _("Invalid backup: %(reason)s")
 
 
 class SwiftConnectionFailed(CinderException):
-    message = _("Connection to swift failed") + ": %(reason)s"
+    message = _("Connection to swift failed: %(reason)s")
+
+
+class TransferNotFound(NotFound):
+    message = _("Transfer %(transfer_id)s could not be found.")
+
+
+class VolumeMigrationFailed(CinderException):
+    message = _("Volume migration failed: %(reason)s")
+
+
+class SSHInjectionThreat(CinderException):
+    message = _("SSH command injection detected: %(command)s")
+
+
+class CoraidException(CinderException):
+    message = _('Coraid Cinder Driver exception.')
+
+
+class CoraidJsonEncodeFailure(CoraidException):
+    message = _('Failed to encode json data.')
+
+
+class CoraidESMBadCredentials(CoraidException):
+    message = _('Login on ESM failed.')
+
+
+class CoraidESMReloginFailed(CoraidException):
+    message = _('Relogin on ESM failed.')
+
+
+class CoraidESMBadGroup(CoraidException):
+    message = _('Group with name "%(group_name)s" not found.')
+
+
+class CoraidESMConfigureError(CoraidException):
+    message = _('ESM configure request failed: %(message)s.')
+
+
+class CoraidESMNotAvailable(CoraidException):
+    message = _('Coraid ESM not available with reason: %(reason)s.')
+
+
+class QoSSpecsExists(Duplicate):
+    message = _("QoS Specs %(specs_id)s already exists.")
+
+
+class QoSSpecsCreateFailed(CinderException):
+    message = _("Failed to create qos_specs: "
+                "%(name)s with specs %(qos_specs)s.")
+
+
+class QoSSpecsUpdateFailed(CinderException):
+    message = _("Failed to update qos_specs: "
+                "%(specs_id)s with specs %(qos_specs)s.")
+
+
+class QoSSpecsNotFound(NotFound):
+    message = _("No such QoS spec %(specs_id)s.")
+
+
+class QoSSpecsAssociateFailed(CinderException):
+    message = _("Failed to associate qos_specs: "
+                "%(specs_id)s with type %(type_id)s.")
+
+
+class QoSSpecsDisassociateFailed(CinderException):
+    message = _("Failed to disassociate qos_specs: "
+                "%(specs_id)s with type %(type_id)s.")
+
+
+class QoSSpecsKeyNotFound(NotFound):
+    message = _("QoS spec %(specs_id)s has no spec with "
+                "key %(specs_key)s.")
+
+
+class InvalidQoSSpecs(Invalid):
+    message = _("Invalid qos specs: %(reason)s")
+
+
+class QoSSpecsInUse(CinderException):
+    message = _("QoS Specs %(specs_id)s is still associated with entities.")
+
+
+class KeyManagerError(CinderException):
+    msg_fmt = _("key manager error: %(reason)s")
