@@ -1,3 +1,4 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # Copyright (c) 2013 The Johns Hopkins University/Applied Physics Laboratory
 # All Rights Reserved.
 #
@@ -13,9 +14,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from castellan import options as castellan_opts
-from oslo_config import cfg
+from oslo.config import cfg
+
+from cinder.openstack.common import importutils
+
+keymgr_opts = [
+    cfg.StrOpt('api_class',
+               default='cinder.keymgr.conf_key_mgr.ConfKeyManager',
+               help='The full class name of the key manager API class'),
+]
 
 CONF = cfg.CONF
+CONF.register_opts(keymgr_opts, group='keymgr')
 
-castellan_opts.set_defaults(CONF)
+
+def API():
+    cls = importutils.import_class(CONF.keymgr.api_class)
+    return cls()
