@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright (c) 2013 The Johns Hopkins University/Applied Physics Laboratory
 # All Rights Reserved.
 #
@@ -23,13 +25,12 @@ from Java.
 
 import abc
 
-import six
 
-
-@six.add_metaclass(abc.ABCMeta)
 class Key(object):
     """Base class to represent all keys."""
 
+    __metaclass__ = abc.ABCMeta
+
     @abc.abstractmethod
     def get_algorithm(self):
         """Returns the key's algorithm.
@@ -54,7 +55,9 @@ class Key(object):
 
 
 class SymmetricKey(Key):
-    """This class represents symmetric keys."""
+    """
+    This class represents symmetric keys
+    """
 
     def __init__(self, alg, key):
         """Create a new SymmetricKey object.
