@@ -1,3 +1,4 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # Copyright (c) 2013 The Johns Hopkins University/Applied Physics Laboratory
 # All Rights Reserved.
 #
@@ -21,7 +22,8 @@ from cinder.keymgr import key_mgr
 
 
 class NotImplementedKeyManager(key_mgr.KeyManager):
-    """Key Manager interface that raises NotImplementedError"""
+    """Key Manager Interface that raises NotImplementedError for all operations
+    """
 
     def create_key(self, ctxt, algorithm='AES', length=256, expiration=None,
                    **kwargs):
