@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010 OpenStack Foundation
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -24,12 +24,12 @@ Scheduler base class that all Schedulers should inherit from
 from oslo.config import cfg
 
 from cinder import db
-from cinder import flags
 from cinder.openstack.common import importutils
 from cinder.openstack.common import timeutils
 from cinder import utils
 from cinder.volume import rpcapi as volume_rpcapi
 
+
 scheduler_driver_opts = [
     cfg.StrOpt('scheduler_host_manager',
                default='cinder.scheduler.host_manager.HostManager',
@@ -39,8 +39,8 @@ scheduler_driver_opts = [
                help='Maximum number of attempts to schedule an volume'),
 ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(scheduler_driver_opts)
+CONF = cfg.CONF
+CONF.register_opts(scheduler_driver_opts)
 
 
 def volume_update_db(context, volume_id, host):
@@ -58,7 +58,7 @@ class Scheduler(object):
 
     def __init__(self):
         self.host_manager = importutils.import_object(
-            FLAGS.scheduler_host_manager)
+            CONF.scheduler_host_manager)
         self.volume_rpcapi = volume_rpcapi.VolumeAPI()
 
     def get_host_list(self):
@@ -84,6 +84,10 @@ class Scheduler(object):
                 for service in services
                 if utils.service_is_up(service)]
 
+    def host_passes_filters(self, context, volume_id, host, filter_properties):
+        """Check if the specified host passes the filters."""
+        raise NotImplementedError(_("Must implement host_passes_filters"))
+
     def schedule(self, context, topic, method, *_args, **_kwargs):
         """Must override schedule method for scheduler to work."""
         raise NotImplementedError(_("Must implement a fallback schedule"))
