@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010 OpenStack Foundation
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -21,10 +21,10 @@
 Simple Scheduler
 """
 
+from oslo.config import cfg
+
 from cinder import db
 from cinder import exception
-from cinder import flags
-from cinder.openstack.common import cfg
 from cinder.scheduler import chance
 from cinder.scheduler import driver
 from cinder import utils
@@ -33,46 +33,58 @@ from cinder import utils
 simple_scheduler_opts = [
     cfg.IntOpt("max_gigabytes",
                default=10000,
-               help="maximum number of volume gigabytes to allow per host"),
-    ]
+               help="maximum number of volume gigabytes to allow per host"), ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(simple_scheduler_opts)
+CONF = cfg.CONF
+CONF.register_opts(simple_scheduler_opts)
 
 
 class SimpleScheduler(chance.ChanceScheduler):
     """Implements Naive Scheduler that tries to find least loaded host."""
 
-    def schedule_create_volume(self, context, volume_id, **_kwargs):
+    def schedule_create_volume(self, context, request_spec, filter_properties):
         """Picks a host that is up and has the fewest volumes."""
         elevated = context.elevated()
 
-        volume_ref = db.volume_get(context, volume_id)
-        availability_zone = volume_ref.get('availability_zone')
+        volume_id = request_spec.get('volume_id')
+        snapshot_id = request_spec.get('snapshot_id')
+        image_id = request_spec.get('image_id')
+        volume_properties = request_spec.get('volume_properties')
+        volume_size = volume_properties.get('size')
+        availability_zone = volume_properties.get('availability_zone')
 
         zone, host = None, None
         if availability_zone:
             zone, _x, host = availability_zone.partition(':')
         if host and context.is_admin:
-            service = db.service_get_by_args(elevated, host, 'cinder-volume')
+            topic = CONF.volume_topic
+            service = db.service_get_by_args(elevated, host, topic)
             if not utils.service_is_up(service):
                 raise exception.WillNotSchedule(host=host)
-            driver.cast_to_volume_host(context, host, 'create_volume',
-                    volume_id=volume_id, **_kwargs)
+            updated_volume = driver.volume_update_db(context, volume_id, host)
+            self.volume_rpcapi.create_volume(context, updated_volume, host,
+                                             request_spec, filter_properties,
+                                             snapshot_id=snapshot_id,
+                                             image_id=image_id)
             return None
 
         results = db.service_get_all_volume_sorted(elevated)
         if zone:
-            results = [(service, gigs) for (service, gigs) in results
-                       if service['availability_zone'] == zone]
+            results = [(s, gigs) for (s, gigs) in results
+                       if s['availability_zone'] == zone]
         for result in results:
             (service, volume_gigabytes) = result
-            if volume_gigabytes + volume_ref['size'] > FLAGS.max_gigabytes:
+            if volume_gigabytes + volume_size > CONF.max_gigabytes:
                 msg = _("Not enough allocatable volume gigabytes remaining")
                 raise exception.NoValidHost(reason=msg)
             if utils.service_is_up(service) and not service['disabled']:
-                driver.cast_to_volume_host(context, service['host'],
-                        'create_volume', volume_id=volume_id, **_kwargs)
+                updated_volume = driver.volume_update_db(context, volume_id,
+                                                         service['host'])
+                self.volume_rpcapi.create_volume(context, updated_volume,
+                                                 service['host'], request_spec,
+                                                 filter_properties,
+                                                 snapshot_id=snapshot_id,
+                                                 image_id=image_id)
                 return None
         msg = _("Is the appropriate service running?")
         raise exception.NoValidHost(reason=msg)
