@@ -1,7 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright (c) 2011 Zadara Storage Inc.
-# Copyright (c) 2011 OpenStack LLC.
+# Copyright (c) 2011 OpenStack Foundation
 # Copyright 2011 University of Southern California
 # All Rights Reserved.
 #
@@ -21,6 +21,9 @@ from lxml import etree
 import webob
 
 from cinder.api.contrib import types_extra_specs
+from cinder import exception
+from cinder.openstack.common.notifier import api as notifier_api
+from cinder.openstack.common.notifier import test_notifier
 from cinder import test
 from cinder.tests.api import fakes
 import cinder.wsgi
@@ -43,13 +46,16 @@ def delete_volume_type_extra_specs(context, volume_type_id, key):
     pass
 
 
+def delete_volume_type_extra_specs_not_found(context, volume_type_id, key):
+    raise exception.VolumeTypeExtraSpecsNotFound("Not Found")
+
+
 def stub_volume_type_extra_specs():
-    specs = {
-            "key1": "value1",
-            "key2": "value2",
-            "key3": "value3",
-            "key4": "value4",
-            "key5": "value5"}
+    specs = {"key1": "value1",
+             "key2": "value2",
+             "key3": "value3",
+             "key4": "value4",
+             "key5": "value5"}
     return specs
 
 
@@ -61,9 +67,19 @@ class VolumeTypesExtraSpecsTest(test.TestCase):
 
     def setUp(self):
         super(VolumeTypesExtraSpecsTest, self).setUp()
+        self.flags(connection_type='fake',
+                   host='fake',
+                   notification_driver=[test_notifier.__name__])
         self.stubs.Set(cinder.db, 'volume_type_get', volume_type_get)
         self.api_path = '/v2/fake/os-volume-types/1/extra_specs'
         self.controller = types_extra_specs.VolumeTypeExtraSpecsController()
+        """to reset notifier drivers left over from other api/contrib tests"""
+        notifier_api._reset_drivers()
+        test_notifier.NOTIFICATIONS = []
+
+    def tearDown(self):
+        notifier_api._reset_drivers()
+        super(VolumeTypesExtraSpecsTest, self).tearDown()
 
     def test_index(self):
         self.stubs.Set(cinder.db, 'volume_type_extra_specs_get',
@@ -104,8 +120,18 @@ class VolumeTypesExtraSpecsTest(test.TestCase):
         self.stubs.Set(cinder.db, 'volume_type_extra_specs_delete',
                        delete_volume_type_extra_specs)
 
+        self.assertEqual(len(test_notifier.NOTIFICATIONS), 0)
         req = fakes.HTTPRequest.blank(self.api_path + '/key5')
         self.controller.delete(req, 1, 'key5')
+        self.assertEqual(len(test_notifier.NOTIFICATIONS), 1)
+
+    def test_delete_not_found(self):
+        self.stubs.Set(cinder.db, 'volume_type_extra_specs_delete',
+                       delete_volume_type_extra_specs_not_found)
+
+        req = fakes.HTTPRequest.blank(self.api_path + '/key6')
+        self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
+                          req, 1, 'key6')
 
     def test_create(self):
         self.stubs.Set(cinder.db,
@@ -113,8 +139,10 @@ class VolumeTypesExtraSpecsTest(test.TestCase):
                        return_create_volume_type_extra_specs)
         body = {"extra_specs": {"key1": "value1"}}
 
+        self.assertEqual(len(test_notifier.NOTIFICATIONS), 0)
         req = fakes.HTTPRequest.blank(self.api_path)
         res_dict = self.controller.create(req, 1, body)
+        self.assertEqual(len(test_notifier.NOTIFICATIONS), 1)
 
         self.assertEqual('value1', res_dict['extra_specs']['key1'])
 
@@ -124,8 +152,10 @@ class VolumeTypesExtraSpecsTest(test.TestCase):
                        return_create_volume_type_extra_specs)
         body = {"key1": "value1"}
 
+        self.assertEqual(len(test_notifier.NOTIFICATIONS), 0)
         req = fakes.HTTPRequest.blank(self.api_path + '/key1')
         res_dict = self.controller.update(req, 1, 'key1', body)
+        self.assertEqual(len(test_notifier.NOTIFICATIONS), 1)
 
         self.assertEqual('value1', res_dict['key1'])
 
@@ -188,14 +218,13 @@ class VolumeTypeExtraSpecsSerializerTest(test.TestCase):
         extra_specs = stub_volume_type_extra_specs()
         text = serializer.serialize(dict(extra_specs=extra_specs))
 
-        print text
         tree = etree.fromstring(text)
 
         self.assertEqual('extra_specs', tree.tag)
         self.assertEqual(len(extra_specs), len(tree))
         seen = set(extra_specs.keys())
         for child in tree:
-            self.assertTrue(child.tag in seen)
+            self.assertIn(child.tag, seen)
             self.assertEqual(extra_specs[child.tag], child.text)
             seen.remove(child.tag)
         self.assertEqual(len(seen), 0)
@@ -206,7 +235,6 @@ class VolumeTypeExtraSpecsSerializerTest(test.TestCase):
         exemplar = dict(key1='value1')
         text = serializer.serialize(exemplar)
 
-        print text
         tree = etree.fromstring(text)
 
         self.assertEqual('key1', tree.tag)
