@@ -1,12 +1,24 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
+# Licensed under the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License. You may obtain
+# a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+# License for the specific language governing permissions and limitations
+# under the License.
+
 import inspect
 import webob
 
+from cinder.api.openstack import wsgi
 from cinder import exception
 from cinder import test
-from cinder.api.openstack import wsgi
-from cinder.tests.api.openstack import fakes
+from cinder.tests.api import fakes
 
 
 class RequestTest(test.TestCase):
@@ -75,6 +87,60 @@ class RequestTest(test.TestCase):
         result = request.best_match_content_type()
         self.assertEqual(result, "application/json")
 
+    def test_best_match_language(self):
+        # Test that we are actually invoking language negotiation by webob
+        request = wsgi.Request.blank('/')
+        accepted = 'unknown-lang'
+        request.headers = {'Accept-Language': accepted}
+
+        def fake_best_match(self, offers, default_match=None):
+            # Match would return None, if requested lang is not found
+            return None
+
+        self.stubs.SmartSet(request.accept_language,
+                            'best_match', fake_best_match)
+
+        self.assertEqual(request.best_match_language(), None)
+        # If accept-language is not included or empty, match should be None
+        request.headers = {'Accept-Language': ''}
+        self.assertEqual(request.best_match_language(), None)
+        request.headers.pop('Accept-Language')
+        self.assertEqual(request.best_match_language(), None)
+
+    def test_cache_and_retrieve_resources(self):
+        request = wsgi.Request.blank('/foo')
+        # Test that trying to retrieve a cached object on
+        # an empty cache fails gracefully
+        self.assertIsNone(request.cached_resource())
+        self.assertIsNone(request.cached_resource_by_id('r-0'))
+
+        resources = []
+        for x in xrange(3):
+            resources.append({'id': 'r-%s' % x})
+
+        # Cache an empty list of resources using the default name
+        request.cache_resource([])
+        self.assertEqual({}, request.cached_resource())
+        self.assertIsNone(request.cached_resource('r-0'))
+        # Cache some resources
+        request.cache_resource(resources[:2])
+        # Cache  one resource
+        request.cache_resource(resources[2])
+        # Cache  a different resource name
+        other_resource = {'id': 'o-0'}
+        request.cache_resource(other_resource, name='other-resource')
+
+        self.assertEqual(resources[0], request.cached_resource_by_id('r-0'))
+        self.assertEqual(resources[1], request.cached_resource_by_id('r-1'))
+        self.assertEqual(resources[2], request.cached_resource_by_id('r-2'))
+        self.assertIsNone(request.cached_resource_by_id('r-3'))
+        self.assertEqual({'r-0': resources[0],
+                          'r-1': resources[1],
+                          'r-2': resources[2]}, request.cached_resource())
+        self.assertEqual(other_resource,
+                         request.cached_resource_by_id('o-0',
+                                                       name='other-resource'))
+
 
 class ActionDispatcherTest(test.TestCase):
     def test_dispatch(self):
@@ -215,7 +281,7 @@ class ResourceTest(test.TestCase):
         expected = 'off'
         self.assertEqual(actual, expected)
 
-    def test_get_method_unknown_controller_action(self):
+    def test_get_method_undefined_controller_action(self):
         class Controller(object):
             def index(self, req, pants=None):
                 return pants
@@ -441,10 +507,9 @@ class ResourceTest(test.TestCase):
 
         extended = ControllerExtended()
         resource.register_actions(extended)
-        self.assertEqual({
-                'fooAction': extended._action_foo,
-                'barAction': extended._action_bar,
-                }, resource.wsgi_actions)
+        self.assertEqual({'fooAction': extended._action_foo,
+                          'barAction': extended._action_bar, },
+                         resource.wsgi_actions)
 
     def test_register_extensions(self):
         class Controller(object):
@@ -765,7 +830,7 @@ class ResponseObjectTest(test.TestCase):
         robj = wsgi.ResponseObject({})
         robj['Header'] = 'foo'
         del robj['hEADER']
-        self.assertFalse('header' in robj.headers)
+        self.assertNotIn('header', robj.headers)
 
     def test_header_isolation(self):
         robj = wsgi.ResponseObject({})
