@@ -1,7 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,22 +18,25 @@
 
 import iso8601
 from lxml import etree
+from oslo.config import cfg
 import webob
 
 from cinder.api.v1 import router
 from cinder.api import xmlutil
-from cinder import flags
 from cinder.openstack.common import jsonutils
 from cinder import test
 
-FLAGS = flags.FLAGS
+
 NS = "{http://docs.openstack.org/common/api/v1.0}"
 
 
+CONF = cfg.CONF
+
+
 class ExtensionTestCase(test.TestCase):
     def setUp(self):
         super(ExtensionTestCase, self).setUp()
-        ext_list = FLAGS.osapi_volume_extension[:]
+        ext_list = CONF.osapi_volume_extension[:]
         fox = ('cinder.tests.api.extensions.foxinsocks.Foxinsocks')
         if fox not in ext_list:
             ext_list.append(fox)
@@ -116,7 +119,7 @@ class ExtensionControllerTest(ExtensionTestCase):
 
         # Make sure we have all the extensions, extras extensions being OK.
         exts = root.findall('{0}extension'.format(NS))
-        self.assert_(len(exts) >= len(self.ext_list))
+        self.assertTrue(len(exts) >= len(self.ext_list))
 
         # Make sure that at least Fox in Sox is correct.
         (fox_ext, ) = [x for x in exts if x.get('alias') == 'FOXNSOX']
