@@ -21,7 +21,6 @@ import webob
 from cinder.api.v2 import snapshots
 from cinder import db
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
 from cinder import test
 from cinder.tests.api import fakes
@@ -29,7 +28,6 @@ from cinder.tests.api.v2 import stubs
 from cinder import volume
 
 
-FLAGS = flags.FLAGS
 LOG = logging.getLogger(__name__)
 
 UUID = '00000000-0000-0000-0000-000000000001'
@@ -103,7 +101,7 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/snapshots')
         resp_dict = self.controller.create(req, body)
 
-        self.assertTrue('snapshot' in resp_dict)
+        self.assertIn('snapshot', resp_dict)
         self.assertEqual(resp_dict['snapshot']['name'],
                          snapshot_name)
         self.assertEqual(resp_dict['snapshot']['description'],
@@ -125,7 +123,7 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/snapshots')
         resp_dict = self.controller.create(req, body)
 
-        self.assertTrue('snapshot' in resp_dict)
+        self.assertIn('snapshot', resp_dict)
         self.assertEqual(resp_dict['snapshot']['name'],
                          snapshot_name)
         self.assertEqual(resp_dict['snapshot']['description'],
@@ -166,7 +164,7 @@ class SnapshotApiTest(test.TestCase):
                 'metadata': {},
             }
         }
-        self.assertEquals(expected, res_dict)
+        self.assertEqual(expected, res_dict)
 
     def test_snapshot_update_missing_body(self):
         body = {}
@@ -211,7 +209,7 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % UUID)
         resp_dict = self.controller.show(req, UUID)
 
-        self.assertTrue('snapshot' in resp_dict)
+        self.assertIn('snapshot', resp_dict)
         self.assertEqual(resp_dict['snapshot']['id'], UUID)
 
     def test_snapshot_show_invalid_id(self):
@@ -226,7 +224,7 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/snapshots/detail')
         resp_dict = self.controller.detail(req)
 
-        self.assertTrue('snapshots' in resp_dict)
+        self.assertIn('snapshots', resp_dict)
         resp_snapshots = resp_dict['snapshots']
         self.assertEqual(len(resp_snapshots), 1)
 
@@ -260,7 +258,7 @@ class SnapshotApiTest(test.TestCase):
         resp = self.controller.index(req)
         self.assertEqual(len(resp['snapshots']), 2)
         for snapshot in resp['snapshots']:
-            self.assertEquals(snapshot['status'], 'available')
+            self.assertEqual(snapshot['status'], 'available')
         # no match
         req = fakes.HTTPRequest.blank('/v2/snapshots?status=error')
         resp = self.controller.index(req)
@@ -313,7 +311,7 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/snapshots?name=backup2')
         resp = self.controller.index(req)
         self.assertEqual(len(resp['snapshots']), 1)
-        self.assertEquals(resp['snapshots'][0]['name'], 'backup2')
+        self.assertEqual(resp['snapshots'][0]['name'], 'backup2')
         # filter no match
         req = fakes.HTTPRequest.blank('/v2/snapshots?name=backup4')
         resp = self.controller.index(req)
@@ -324,7 +322,7 @@ class SnapshotApiTest(test.TestCase):
                                       use_admin_context=True)
         res = self.controller.index(req)
 
-        self.assertTrue('snapshots' in res)
+        self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
     def test_list_snapshots_with_limit_and_offset(self):
@@ -344,7 +342,7 @@ class SnapshotApiTest(test.TestCase):
                                           use_admin_context=is_admin)
             res = self.controller.index(req)
 
-            self.assertTrue('snapshots' in res)
+            self.assertIn('snapshots', res)
             self.assertEqual(1, len(res['snapshots']))
             self.assertEqual(2, res['snapshots'][0]['id'])
 
@@ -357,19 +355,19 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/fake/snapshots?all_tenants=1',
                                       use_admin_context=True)
         res = self.controller.index(req)
-        self.assertTrue('snapshots' in res)
+        self.assertIn('snapshots', res)
         self.assertEqual(3, len(res['snapshots']))
 
     def test_all_tenants_non_admin_gets_all_tenants(self):
         req = fakes.HTTPRequest.blank('/v2/fake/snapshots?all_tenants=1')
         res = self.controller.index(req)
-        self.assertTrue('snapshots' in res)
+        self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
     def test_non_admin_get_by_project(self):
         req = fakes.HTTPRequest.blank('/v2/fake/snapshots')
         res = self.controller.index(req)
-        self.assertTrue('snapshots' in res)
+        self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
     def _create_snapshot_bad_body(self, body):
@@ -413,7 +411,6 @@ class SnapshotSerializerTest(test.TestCase):
         )
         text = serializer.serialize(dict(snapshot=raw_snapshot))
 
-        print text
         tree = etree.fromstring(text)
 
         self._verify_snapshot(raw_snapshot, tree)
@@ -442,7 +439,6 @@ class SnapshotSerializerTest(test.TestCase):
         ]
         text = serializer.serialize(dict(snapshots=raw_snapshots))
 
-        print text
         tree = etree.fromstring(text)
 
         self.assertEqual('snapshots', tree.tag)
