@@ -1,7 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright 2011 Justin Santa Barbara
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,7 +23,6 @@ import datetime
 import uuid
 
 from cinder import exception
-from cinder import flags
 import cinder.image.glance
 from cinder.openstack.common import log as logging
 
@@ -31,9 +30,6 @@ from cinder.openstack.common import log as logging
 LOG = logging.getLogger(__name__)
 
 
-FLAGS = flags.FLAGS
-
-
 class _FakeImageService(object):
     """Mock (fake) image service for unit testing."""
 
@@ -41,7 +37,7 @@ class _FakeImageService(object):
         self.images = {}
         # NOTE(justinsb): The OpenStack API can't upload an image?
         # So, make sure we've got one..
-        timestamp = datetime.datetime(2011, 01, 01, 01, 02, 03)
+        timestamp = datetime.datetime(2011, 1, 1, 1, 2, 3)
 
         image1 = {'id': '155d900f-4e14-4e4c-a73d-069cbf4541e6',
                   'name': 'fakeimage123456',
