@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,21 +17,20 @@
 
 
 import datetime
-import random
-import time
 
 import glanceclient.exc
+import glanceclient.v2.client
+from glanceclient.v2.client import Client as glanceclient_v2
+from oslo.config import cfg
 
 from cinder import context
 from cinder import exception
-from cinder import flags
 from cinder.image import glance
 from cinder import test
 from cinder.tests.glance import stubs as glance_stubs
-from glanceclient.v2.client import Client as glanceclient_v2
 
 
-FLAGS = flags.FLAGS
+CONF = cfg.CONF
 
 
 class NullWriter(object):
@@ -78,8 +77,7 @@ class TestGlanceSerializer(test.TestCase):
 
 
 class TestGlanceImageService(test.TestCase):
-    """
-    Tests the Glance image service.
+    """Tests the Glance image service.
 
     At a high level, the translations involved are:
 
@@ -110,7 +108,7 @@ class TestGlanceImageService(test.TestCase):
         self.stubs.Set(glance.time, 'sleep', lambda s: None)
 
     def _create_image_service(self, client):
-        def _fake_create_glance_client(context, host, port, use_ssl, version):
+        def _fake_create_glance_client(context, netloc, use_ssl, version):
             return client
 
         self.stubs.Set(glance,
@@ -166,7 +164,8 @@ class TestGlanceImageService(test.TestCase):
         self.assertDictMatch(image_metas[0], expected)
 
     def test_create_without_instance_id(self):
-        """
+        """Test Creating images without instance_id.
+
         Ensure we can create an image without having to specify an
         instance_id. Public images are an example of an image not tied to an
         instance.
@@ -201,8 +200,8 @@ class TestGlanceImageService(test.TestCase):
         image_id = self.service.create(self.context, fixture)['id']
 
         self.assertNotEquals(None, image_id)
-        self.assertEquals(num_images + 1,
-                          len(self.service.detail(self.context)))
+        self.assertEqual(num_images + 1,
+                         len(self.service.detail(self.context)))
 
     def test_create_and_show_non_existing_image(self):
         fixture = self._make_fixture(name='test image')
@@ -242,7 +241,7 @@ class TestGlanceImageService(test.TestCase):
             ids.append(self.service.create(self.context, fixture)['id'])
 
         image_metas = self.service.detail(self.context, marker=ids[1])
-        self.assertEquals(len(image_metas), 8)
+        self.assertEqual(len(image_metas), 8)
         i = 2
         for meta in image_metas:
             expected = {
@@ -276,7 +275,7 @@ class TestGlanceImageService(test.TestCase):
             ids.append(self.service.create(self.context, fixture)['id'])
 
         image_metas = self.service.detail(self.context, limit=5)
-        self.assertEquals(len(image_metas), 5)
+        self.assertEqual(len(image_metas), 5)
 
     def test_detail_default_limit(self):
         fixtures = []
@@ -299,7 +298,7 @@ class TestGlanceImageService(test.TestCase):
             ids.append(self.service.create(self.context, fixture)['id'])
 
         image_metas = self.service.detail(self.context, marker=ids[3], limit=5)
-        self.assertEquals(len(image_metas), 5)
+        self.assertEqual(len(image_metas), 5)
         i = 4
         for meta in image_metas:
             expected = {
@@ -337,13 +336,12 @@ class TestGlanceImageService(test.TestCase):
     def test_update(self):
         fixture = self._make_fixture(name='test image')
         image = self.service.create(self.context, fixture)
-        print image
         image_id = image['id']
         fixture['name'] = 'new image name'
         self.service.update(self.context, image_id, fixture)
 
         new_image_data = self.service.show(self.context, image_id)
-        self.assertEquals('new image name', new_image_data['name'])
+        self.assertEqual('new image name', new_image_data['name'])
 
     def test_delete(self):
         fixture1 = self._make_fixture(name='test image 1')
@@ -351,7 +349,7 @@ class TestGlanceImageService(test.TestCase):
         fixtures = [fixture1, fixture2]
 
         num_images = len(self.service.detail(self.context))
-        self.assertEquals(0, num_images)
+        self.assertEqual(0, num_images)
 
         ids = []
         for fixture in fixtures:
@@ -359,12 +357,12 @@ class TestGlanceImageService(test.TestCase):
             ids.append(new_id)
 
         num_images = len(self.service.detail(self.context))
-        self.assertEquals(2, num_images)
+        self.assertEqual(2, num_images)
 
         self.service.delete(self.context, ids[0])
 
         num_images = len(self.service.detail(self.context))
-        self.assertEquals(1, num_images)
+        self.assertEqual(1, num_images)
 
     def test_show_passes_through_to_client(self):
         fixture = self._make_fixture(name='image1', is_public=True)
@@ -529,7 +527,7 @@ class TestGlanceImageService(test.TestCase):
         image_id = self.service.create(self.context, fixture)['id']
         (service, same_id) = glance.get_remote_image_service(self.context,
                                                              image_id)
-        self.assertEquals(same_id, image_id)
+        self.assertEqual(same_id, image_id)
 
     def test_glance_client_image_ref(self):
         fixture = self._make_fixture(name='test image')
@@ -537,13 +535,18 @@ class TestGlanceImageService(test.TestCase):
         image_url = 'http://something-less-likely/%s' % image_id
         (service, same_id) = glance.get_remote_image_service(self.context,
                                                              image_url)
-        self.assertEquals(same_id, image_id)
-        self.assertEquals(service._client.host,
-                          'something-less-likely')
+        self.assertEqual(same_id, image_id)
+        self.assertEqual(service._client.netloc, 'something-less-likely')
+        for ipv6_url in ('[::1]', '::1', '[::1]:444'):
+            image_url = 'http://%s/%s' % (ipv6_url, image_id)
+            (service, same_id) = glance.get_remote_image_service(self.context,
+                                                                 image_url)
+            self.assertEqual(same_id, image_id)
+            self.assertEqual(service._client.netloc, ipv6_url)
 
 
 class TestGlanceClientVersion(test.TestCase):
-    """Tests the version of the glance client generated"""
+    """Tests the version of the glance client generated."""
     def setUp(self):
         super(TestGlanceClientVersion, self).setUp()
 
@@ -561,28 +564,28 @@ class TestGlanceClientVersion(test.TestCase):
             pass
 
     def test_glance_version_by_flag(self):
-        """Test glance version set by flag is honoured"""
+        """Test glance version set by flag is honoured."""
         client_wrapper_v1 = glance.GlanceClientWrapper('fake', 'fake_host',
                                                        9292)
-        self.assertEquals(client_wrapper_v1.client.__module__,
-                          'glanceclient.v1.client')
+        self.assertEqual(client_wrapper_v1.client.__module__,
+                         'glanceclient.v1.client')
         self.flags(glance_api_version=2)
         client_wrapper_v2 = glance.GlanceClientWrapper('fake', 'fake_host',
                                                        9292)
-        self.assertEquals(client_wrapper_v2.client.__module__,
-                          'glanceclient.v2.client')
-        FLAGS.reset()
+        self.assertEqual(client_wrapper_v2.client.__module__,
+                         'glanceclient.v2.client')
+        CONF.reset()
 
     def test_glance_version_by_arg(self):
         """Test glance version set by arg to GlanceClientWrapper"""
         client_wrapper_v1 = glance.GlanceClientWrapper('fake', 'fake_host',
                                                        9292, version=1)
-        self.assertEquals(client_wrapper_v1.client.__module__,
-                          'glanceclient.v1.client')
+        self.assertEqual(client_wrapper_v1.client.__module__,
+                         'glanceclient.v1.client')
         client_wrapper_v2 = glance.GlanceClientWrapper('fake', 'fake_host',
                                                        9292, version=2)
-        self.assertEquals(client_wrapper_v2.client.__module__,
-                          'glanceclient.v2.client')
+        self.assertEqual(client_wrapper_v2.client.__module__,
+                         'glanceclient.v2.client')
 
 
 def _create_failing_glance_client(info):
@@ -595,3 +598,63 @@ def _create_failing_glance_client(info):
             return {}
 
     return MyGlanceStubClient()
+
+
+class TestGlanceImageServiceClient(test.TestCase):
+
+    def setUp(self):
+        super(TestGlanceImageServiceClient, self).setUp()
+        self.context = context.RequestContext('fake', 'fake', auth_token=True)
+        self.stubs.Set(glance.time, 'sleep', lambda s: None)
+
+    def test_create_glance_client(self):
+        self.flags(auth_strategy='keystone')
+        self.flags(glance_request_timeout=60)
+
+        class MyGlanceStubClient(object):
+            def __init__(inst, version, *args, **kwargs):
+                self.assertEqual('1', version)
+                self.assertEqual("http://fake_host:9292", args[0])
+                self.assertEqual(True, kwargs['token'])
+                self.assertEqual(60, kwargs['timeout'])
+
+        self.stubs.Set(glance.glanceclient, 'Client', MyGlanceStubClient)
+        client = glance._create_glance_client(self.context, 'fake_host:9292',
+                                              False)
+        self.assertIsInstance(client, MyGlanceStubClient)
+
+    def test_create_glance_client_auth_strategy_is_not_keystone(self):
+        self.flags(auth_strategy='noauth')
+        self.flags(glance_request_timeout=60)
+
+        class MyGlanceStubClient(object):
+            def __init__(inst, version, *args, **kwargs):
+                self.assertEqual('1', version)
+                self.assertEqual('http://fake_host:9292', args[0])
+                self.assertNotIn('token', kwargs)
+                self.assertEqual(60, kwargs['timeout'])
+
+        self.stubs.Set(glance.glanceclient, 'Client', MyGlanceStubClient)
+        client = glance._create_glance_client(self.context, 'fake_host:9292',
+                                              False)
+        self.assertIsInstance(client, MyGlanceStubClient)
+
+    def test_create_glance_client_glance_request_default_timeout(self):
+        self.flags(auth_strategy='keystone')
+        self.flags(glance_request_timeout=None)
+
+        class MyGlanceStubClient(object):
+            def __init__(inst, version, *args, **kwargs):
+                self.assertEqual("1", version)
+                self.assertEqual("http://fake_host:9292", args[0])
+                self.assertEqual(True, kwargs['token'])
+                self.assertNotIn('timeout', kwargs)
+
+        self.stubs.Set(glance.glanceclient, 'Client', MyGlanceStubClient)
+        client = glance._create_glance_client(self.context, 'fake_host:9292',
+                                              False)
+        self.assertIsInstance(client, MyGlanceStubClient)
+
+    def tearDown(self):
+        self.stubs.UnsetAll()
+        super(TestGlanceImageServiceClient, self).tearDown()
