@@ -1,4 +1,4 @@
-# Copyright 2011-2012 OpenStack LLC.
+# Copyright 2011-2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,6 +16,8 @@
 Tests For Capacity Weigher.
 """
 
+import testtools
+
 from cinder import context
 from cinder.openstack.common.scheduler.weights import HostWeightHandler
 from cinder import test
@@ -46,8 +48,6 @@ class CapacityWeigherTestCase(test.TestCase):
         self.mox.ResetAll()
         return host_states
 
-    @test.skip_if(not test_utils.is_cinder_installed(),
-                  'Test requires Cinder installed')
     def test_default_of_spreading_first(self):
         hostinfo_list = self._get_all_hosts()
 
@@ -61,8 +61,6 @@ class CapacityWeigherTestCase(test.TestCase):
         self.assertEqual(weighed_host.weight, 921.0)
         self.assertEqual(weighed_host.obj.host, 'host1')
 
-    @test.skip_if(not test_utils.is_cinder_installed(),
-                  'Test requires Cinder installed')
     def test_capacity_weight_multiplier1(self):
         self.flags(capacity_weight_multiplier=-1.0)
         hostinfo_list = self._get_all_hosts()
@@ -77,8 +75,6 @@ class CapacityWeigherTestCase(test.TestCase):
         self.assertEqual(weighed_host.weight, -190.0)
         self.assertEqual(weighed_host.obj.host, 'host4')
 
-    @test.skip_if(not test_utils.is_cinder_installed(),
-                  'Test requires Cinder installed')
     def test_capacity_weight_multiplier2(self):
         self.flags(capacity_weight_multiplier=2.0)
         hostinfo_list = self._get_all_hosts()
