@@ -1,7 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright (c) 2011 Zadara Storage Inc.
-# Copyright (c) 2011 OpenStack LLC.
+# Copyright (c) 2011 OpenStack Foundation
 # Copyright 2011 University of Southern California
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -85,6 +85,26 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
         for key, value in expected_metadata_1.items():
             self.assertEqual(metadata[0][key], value)
 
+    def test_vols_get_glance_metadata(self):
+        ctxt = context.get_admin_context()
+        db.volume_create(ctxt, {'id': '1'})
+        db.volume_create(ctxt, {'id': '2'})
+        db.volume_create(ctxt, {'id': '3'})
+        db.volume_glance_metadata_create(ctxt, '1', 'key1', 'value1')
+        db.volume_glance_metadata_create(ctxt, '2', 'key2', 'value2')
+        db.volume_glance_metadata_create(ctxt, '2', 'key22', 'value22')
+
+        metadata = db.volume_glance_metadata_get_all(ctxt)
+        self.assertEqual(len(metadata), 3)
+        self._assert_metadata_equals('1', 'key1', 'value1', metadata[0])
+        self._assert_metadata_equals('2', 'key2', 'value2', metadata[1])
+        self._assert_metadata_equals('2', 'key22', 'value22', metadata[2])
+
+    def _assert_metadata_equals(self, volume_id, key, value, observed):
+        self.assertEqual(volume_id, observed.volume_id)
+        self.assertEqual(key, observed.key)
+        self.assertEqual(value, observed.value)
+
     def test_vol_delete_glance_metadata(self):
         ctxt = context.get_admin_context()
         db.volume_create(ctxt, {'id': 1})
@@ -109,7 +129,7 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
 
         for meta in db.volume_snapshot_glance_metadata_get(ctxt, 100):
             for (key, value) in expected_meta.items():
-                self.assertEquals(meta[key], value)
+                self.assertEqual(meta[key], value)
 
     def test_vol_glance_metadata_copy_to_volume(self):
         ctxt = context.get_admin_context()
@@ -124,4 +144,4 @@ class VolumeGlanceMetadataTestCase(test.TestCase):
 
         for meta in db.volume_glance_metadata_get(ctxt, 100):
             for (key, value) in expected_meta.items():
-                self.assertEquals(meta[key], value)
+                self.assertEqual(meta[key], value)
