@@ -18,8 +18,12 @@
 
 # Importing full names to not pollute the namespace and cause possible
 # collisions with use of 'from cinder.volume import <foo>' elsewhere.
-import cinder.flags
-import cinder.openstack.common.importutils
 
-API = cinder.openstack.common.importutils.import_class(
-        cinder.flags.FLAGS.volume_api_class)
+
+from cinder.common import config
+import cinder.openstack.common.importutils as import_utils
+
+
+CONF = config.CONF
+
+API = import_utils.import_class(CONF.volume_api_class)
