@@ -697,6 +697,7 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
         try:
             (dest_type, dest_hostname, dest_vg, lvm_type, lvm_mirrors) =\
                 info.split(':')
+            lvm_mirrors = int(lvm_mirrors)
         except ValueError:
             return false_ret
         if (dest_type != 'LVMVolumeDriver' or dest_hostname != self.hostname):
@@ -705,7 +706,7 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
         if dest_vg != self.vg.vg_name:
             vg_list = volutils.get_all_volume_groups()
             vg_dict = \
-                (vg for vg in vg_list if vg['name'] == self.vg.vg_name).next()
+                (vg for vg in vg_list if vg['name'] == dest_vg).next()
             if vg_dict is None:
                 message = ("Destination Volume Group %s does not exist" %
                            dest_vg)
@@ -713,7 +714,9 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
                 return false_ret
 
             helper = 'sudo cinder-rootwrap %s' % CONF.rootwrap_config
-            dest_vg_ref = lvm.LVM(dest_vg, helper, lvm_type, self._execute)
+            dest_vg_ref = lvm.LVM(dest_vg, helper,
+                                  lvm_type=lvm_type,
+                                  executor=self._execute)
             self.remove_export(ctxt, volume)
             self._create_volume(volume['name'],
                                 self._sizestr(volume['size']),
