@@ -603,7 +603,8 @@ class NetAppDirectISCSIDriver(driver.ISCSIDriver):
         if curr_size_bytes != new_size_bytes:
             lun_geometry = self._get_lun_geometry(path)
             if (lun_geometry and lun_geometry.get("max_resize")
-                    and lun_geometry.get("max_resize") >= new_size_bytes):
+                    and int(lun_geometry.get("max_resize")) >=
+                    int(new_size_bytes)):
                 self._do_direct_resize(path, new_size_bytes)
             else:
                 self._do_sub_clone_resize(path, new_size_bytes)
@@ -781,7 +782,11 @@ class NetAppDirectCmodeISCSIDriver(NetAppDirectISCSIDriver):
         self.client.set_api_version(major, minor)
         self.ssc_vols = None
         self.stale_vols = set()
-        ssc_utils.refresh_cluster_ssc(self, self.client, self.vserver)
+
+    def check_for_setup_error(self):
+        """Check that the driver is working and can communicate."""
+        ssc_utils.check_ssc_api_permissions(self.client)
+        super(NetAppDirectCmodeISCSIDriver, self).check_for_setup_error()
 
     def _create_lun_on_eligible_vol(self, name, size, metadata,
                                     extra_specs=None):
@@ -1057,6 +1062,9 @@ class NetAppDirectCmodeISCSIDriver(NetAppDirectISCSIDriver):
 
     def _update_cluster_vol_stats(self, data):
         """Updates vol stats with cluster config."""
+        sync = True if self.ssc_vols is None else False
+        ssc_utils.refresh_cluster_ssc(self, self.client, self.vserver,
+                                      synchronous=sync)
         if self.ssc_vols:
             data['netapp_mirrored'] = 'true'\
                 if self.ssc_vols['mirrored'] else 'false'
@@ -1090,7 +1098,6 @@ class NetAppDirectCmodeISCSIDriver(NetAppDirectISCSIDriver):
                 data['free_capacity_gb'] = 0
         else:
             LOG.warn(_("Cluster ssc is not updated. No volume stats found."))
-        ssc_utils.refresh_cluster_ssc(self, self.client, self.vserver)
 
     @utils.synchronized('update_stale')
     def _update_stale_vols(self, volume=None, reset=False):
@@ -1107,6 +1114,17 @@ class NetAppDirectCmodeISCSIDriver(NetAppDirectISCSIDriver):
         """Refreshes ssc_vols with latest entries."""
         self.ssc_vols = vols
 
+    def delete_volume(self, volume):
+        """Driver entry point for destroying existing volumes."""
+        lun = self.lun_table.get(volume['name'])
+        netapp_vol = None
+        if lun:
+            netapp_vol = lun.get_metadata_property('Volume')
+        super(NetAppDirectCmodeISCSIDriver, self).delete_volume(volume)
+        if netapp_vol:
+            self._update_stale_vols(
+                volume=ssc_utils.NetAppVolume(netapp_vol, self.vserver))
+
 
 class NetAppDirect7modeISCSIDriver(NetAppDirectISCSIDriver):
     """NetApp 7-mode iSCSI volume driver."""
