@@ -17,6 +17,7 @@
 
 import errno
 import os
+import re
 
 from oslo.config import cfg
 
@@ -90,9 +91,25 @@ class RemoteFsDriver(driver.VolumeDriver):
             data['options'] = self.shares[volume['provider_location']]
         return {
             'driver_volume_type': self.driver_volume_type,
-            'data': data
+            'data': data,
+            'mount_point_base': self._get_mount_point_base()
         }
 
+    def _get_mount_point_base(self):
+        """Returns the mount point base for the remote fs.
+
+           This method facilitates returning mount point base
+           for the specific remote fs. Override this method
+           in the respective driver to return the entry to be
+           used while attach/detach using brick in cinder.
+           If not overridden then it returns None without
+           raising exception to continue working for cases
+           when not used with brick.
+        """
+        LOG.debug(_("Driver specific implementation needs to return"
+                    " mount_point_base."))
+        return None
+
     def create_volume(self, volume):
         """Creates a volume.
 
@@ -278,6 +295,11 @@ class RemoteFsDriver(driver.VolumeDriver):
             share_address = share_info[0].strip().decode('unicode_escape')
             share_opts = share_info[1].strip() if len(share_info) > 1 else None
 
+            if not re.match(r'.+:/.+', share_address):
+                LOG.warn("Share %s ignored due to invalid format.  Must be of "
+                         "form address:/export." % share_address)
+                continue
+
             self.shares[share_address] = share_opts
 
         LOG.debug("shares loaded: %s", self.shares)
@@ -372,15 +394,16 @@ class NfsDriver(RemoteFsDriver):
         super(NfsDriver, self).__init__(*args, **kwargs)
         self.configuration.append_config_values(volume_opts)
         root_helper = utils.get_root_helper()
-        base = getattr(self.configuration,
-                       'nfs_mount_point_base',
-                       CONF.nfs_mount_point_base)
+        # base bound to instance is used in RemoteFsConnector.
+        self.base = getattr(self.configuration,
+                            'nfs_mount_point_base',
+                            CONF.nfs_mount_point_base)
         opts = getattr(self.configuration,
                        'nfs_mount_options',
                        CONF.nfs_mount_options)
         self._remotefsclient = remotefs.RemoteFsClient(
             'nfs', root_helper, execute=execute,
-            nfs_mount_point_base=base,
+            nfs_mount_point_base=self.base,
             nfs_mount_options=opts)
 
     def set_execute(self, execute):
@@ -532,3 +555,6 @@ class NfsDriver(RemoteFsDriver):
                               '*snapshot*', mount_point, run_as_root=True)
         total_allocated = float(du.split()[0])
         return total_size, total_available, total_allocated
+
+    def _get_mount_point_base(self):
+        return self.base
