@@ -22,4 +22,6 @@
 """
 
 # Adding imports for backwards compatibility in loading volume_driver.
-from cinder.volume.drivers.san.san import SanISCSIDriver      # noqa
+from hp_lefthand import HpSanISCSIDriver
+from san import SanISCSIDriver
+from solaris import SolarisISCSIDriver
