@@ -1,4 +1,6 @@
-#    Copyright 2012 OpenStack LLC
+#    Copyright 2012 OpenStack Foundation
+#    Copyright (c) 2013 Zelin.io
+#    All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,21 +18,31 @@
 SheepDog Volume Driver.
 
 """
+import os
 import re
+import tempfile
+
+from oslo.config import cfg
 
 from cinder import exception
-from cinder import flags
+from cinder.image import image_utils
 from cinder.openstack.common import log as logging
+from cinder.openstack.common import processutils
+from cinder import units
 from cinder.volume import driver
 
 
 LOG = logging.getLogger(__name__)
-FLAGS = flags.FLAGS
+
+CONF = cfg.CONF
+CONF.import_opt("image_conversion_dir", "cinder.image.image_utils")
 
 
 class SheepdogDriver(driver.VolumeDriver):
     """Executes commands relating to Sheepdog Volumes"""
 
+    VERSION = "1.0.0"
+
     def __init__(self, *args, **kwargs):
         super(SheepdogDriver, self).__init__(*args, **kwargs)
         self.stats_pattern = re.compile(r'[\w\s%]*Total\s(\d+)\s(\d+)*')
@@ -43,12 +55,12 @@ class SheepdogDriver(driver.VolumeDriver):
             #  gives short output, but for compatibility reason we won't
             #  use it and just check if 'running' is in the output.
             (out, err) = self._execute('collie', 'cluster', 'info')
-            if 'running' not in out.split():
+            if 'status: running' not in out:
                 exception_message = (_("Sheepdog is not working: %s") % out)
                 raise exception.VolumeBackendAPIException(
                     data=exception_message)
 
-        except exception.ProcessExecutionError:
+        except processutils.ProcessExecutionError:
             exception_message = _("Sheepdog is not working")
             raise exception.VolumeBackendAPIException(data=exception_message)
 
@@ -70,7 +82,39 @@ class SheepdogDriver(driver.VolumeDriver):
 
     def delete_volume(self, volume):
         """Deletes a logical volume"""
-        self._try_execute('collie', 'vdi', 'delete', volume['name'])
+        self._delete(volume)
+
+    def _ensure_dir_exists(self, tmp_dir):
+        if tmp_dir and not os.path.exists(tmp_dir):
+            os.makedirs(tmp_dir)
+
+    def _resize(self, volume, size=None):
+        if not size:
+            size = int(volume['size']) * units.GiB
+
+        self._try_execute('collie', 'vdi', 'resize',
+                          volume['name'], size)
+
+    def _delete(self, volume):
+        self._try_execute('collie', 'vdi', 'delete',
+                          volume['name'])
+
+    def copy_image_to_volume(self, context, volume, image_service, image_id):
+        # use the image_conversion_dir as a temporary place to save the image
+        conversion_dir = CONF.image_conversion_dir
+        self._ensure_dir_exists(conversion_dir)
+        with tempfile.NamedTemporaryFile(dir=conversion_dir) as tmp:
+            # (wenhao): we don't need to convert to raw for sheepdog.
+            image_utils.fetch_verify_image(context, image_service,
+                                           image_id, tmp.name)
+
+            # remove the image created by import before this function.
+            # see volume/drivers/manager.py:_create_volume
+            self._delete(volume)
+            # convert and store into sheepdog
+            image_utils.convert_image(tmp, 'sheepdog:%s' % volume['name'],
+                                      'raw')
+            self._resize(volume)
 
     def create_snapshot(self, snapshot):
         """Creates a sheepdog snapshot"""
@@ -116,7 +160,7 @@ class SheepdogDriver(driver.VolumeDriver):
             backend_name = self.configuration.safe_get('volume_backend_name')
         stats["volume_backend_name"] = backend_name or 'sheepdog'
         stats['vendor_name'] = 'Open Source'
-        stats['dirver_version'] = '1.0'
+        stats['dirver_version'] = self.VERSION
         stats['storage_protocol'] = 'sheepdog'
         stats['total_capacity_gb'] = 'unknown'
         stats['free_capacity_gb'] = 'unknown'
@@ -130,7 +174,7 @@ class SheepdogDriver(driver.VolumeDriver):
             used = float(m.group(2))
             stats['total_capacity_gb'] = total / (1024 ** 3)
             stats['free_capacity_gb'] = (total - used) / (1024 ** 3)
-        except exception.ProcessExecutionError:
+        except processutils.ProcessExecutionError:
             LOG.exception(_('error refreshing volume stats'))
 
         self._stats = stats
@@ -139,3 +183,27 @@ class SheepdogDriver(driver.VolumeDriver):
         if refresh:
             self._update_volume_stats()
         return self._stats
+
+    def extend_volume(self, volume, new_size):
+        """Extend an Existing Volume."""
+        old_size = volume['size']
+
+        try:
+            size = int(new_size) * units.GiB
+            self._resize(volume, size=size)
+        except Exception:
+            msg = _('Failed to Extend Volume '
+                    '%(volname)s') % {'volname': volume['name']}
+            LOG.error(msg)
+            raise exception.VolumeBackendAPIException(data=msg)
+
+        LOG.debug(_("Extend volume from %(old_size)s GB to %(new_size)s GB."),
+                  {'old_size': old_size, 'new_size': new_size})
+
+    def backup_volume(self, context, backup, backup_service):
+        """Create a new backup from an existing volume."""
+        raise NotImplementedError()
+
+    def restore_backup(self, context, backup, volume, backup_service):
+        """Restore an existing backup to a new or existing volume."""
+        raise NotImplementedError()
