@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2011 Justin Santa Barbara
+# Copyright 2013 SolidFire Inc
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -30,10 +30,10 @@ from oslo.config import cfg
 from cinder import context
 from cinder import exception
 from cinder.openstack.common import log as logging
+from cinder.openstack.common import timeutils
 from cinder.volume.drivers.san.san import SanISCSIDriver
 from cinder.volume import volume_types
 
-VERSION = '1.2'
 LOG = logging.getLogger(__name__)
 
 sf_opts = [
@@ -47,10 +47,19 @@ sf_opts = [
 
     cfg.StrOpt('sf_account_prefix',
                default=socket.gethostname(),
-               help='Create SolidFire accounts with this prefix'), ]
+               help='Create SolidFire accounts with this prefix'),
+
+    cfg.IntOpt('sf_api_port',
+               default=443,
+               help='SolidFire API port. Useful if the device api is behind '
+                    'a proxy on a different port.'), ]
 
 
-class SolidFire(SanISCSIDriver):
+CONF = cfg.CONF
+CONF.register_opts(sf_opts)
+
+
+class SolidFireDriver(SanISCSIDriver):
     """OpenStack driver to enable SolidFire cluster.
 
     Version history:
@@ -59,6 +68,8 @@ class SolidFire(SanISCSIDriver):
 
     """
 
+    VERSION = '1.2.0'
+
     sf_qos_dict = {'slow': {'minIOPS': 100,
                             'maxIOPS': 200,
                             'burstIOPS': 200},
@@ -79,14 +90,14 @@ class SolidFire(SanISCSIDriver):
     GB = math.pow(2, 30)
 
     def __init__(self, *args, **kwargs):
-            super(SolidFire, self).__init__(*args, **kwargs)
+            super(SolidFireDriver, self).__init__(*args, **kwargs)
             self.configuration.append_config_values(sf_opts)
             try:
                 self._update_cluster_status()
             except exception.SolidFireAPIException:
                 pass
 
-    def _issue_api_request(self, method_name, params):
+    def _issue_api_request(self, method_name, params, version='1.0'):
         """All API requests to SolidFire device go through this method.
 
         Simple json-rpc web based API calls.
@@ -99,8 +110,7 @@ class SolidFire(SanISCSIDriver):
                                    'xMaxSnapshotsPerNodeExceeded',
                                    'xMaxClonesPerNodeExceeded']
         host = self.configuration.san_ip
-        # For now 443 is the only port our server accepts requests on
-        port = 443
+        port = self.configuration.sf_api_port
 
         cluster_admin = self.configuration.san_login
         cluster_password = self.configuration.san_password
@@ -111,7 +121,7 @@ class SolidFire(SanISCSIDriver):
         # can't be re-used
         retry_count = 5
         while retry_count > 0:
-            request_id = int(uuid.uuid4())  # just generate a random number
+            request_id = hash(uuid.uuid4())  # just generate a random number
             command = {'method': method_name,
                        'id': request_id}
 
@@ -131,9 +141,10 @@ class SolidFire(SanISCSIDriver):
 
             LOG.debug(_("Payload for SolidFire API call: %s"), payload)
 
+            api_endpoint = '/json-rpc/%s' % version
             connection = httplib.HTTPSConnection(host, port)
             try:
-                connection.request('POST', '/json-rpc/1.0', payload, header)
+                connection.request('POST', api_endpoint, payload, header)
             except Exception as ex:
                 LOG.error(_('Failed to make httplib connection '
                             'SolidFire Cluster: %s (verify san_ip '
@@ -159,7 +170,7 @@ class SolidFire(SanISCSIDriver):
                 data = response.read()
                 try:
                     data = json.loads(data)
-                except (TypeError, ValueError), exc:
+                except (TypeError, ValueError) as exc:
                     connection.close()
                     msg = _("Call to json.loads() raised "
                             "an exception: %s") % exc
@@ -292,13 +303,14 @@ class SolidFire(SanISCSIDriver):
                     iqn = v['iqn']
                     found_volume = True
                     break
-            time.sleep(2)
+            if not found_volume:
+                time.sleep(2)
             iteration_count += 1
 
         if not found_volume:
             LOG.error(_('Failed to retrieve volume SolidFire-'
                         'ID: %s in get_by_account!') % sf_volume_id)
-            raise exception.VolumeNotFound(volume_id=uuid)
+            raise exception.VolumeNotFound(volume_id=sf_volume_id)
 
         model_update = {}
         # NOTE(john-griffith): SF volumes are always at lun 0
@@ -306,7 +318,10 @@ class SolidFire(SanISCSIDriver):
                                              % (iscsi_portal, iqn, 0))
         model_update['provider_auth'] = ('CHAP %s %s'
                                          % (sfaccount['username'],
-                                         chap_secret))
+                                            chap_secret))
+        if not self.configuration.sf_emulate_512:
+            model_update['provider_geometry'] = ('%s %s' % (4096, 4096))
+
         return model_update
 
     def _do_clone_volume(self, src_uuid, src_project_id, v_ref):
@@ -327,7 +342,7 @@ class SolidFire(SanISCSIDriver):
 
         sf_vol = self._get_sf_volume(src_uuid, params)
         if sf_vol is None:
-            raise exception.VolumeNotFound(volume_id=uuid)
+            raise exception.VolumeNotFound(volume_id=src_uuid)
 
         if src_project_id != v_ref['project_id']:
             sfaccount = self._create_sfaccount(v_ref['project_id'])
@@ -361,9 +376,11 @@ class SolidFire(SanISCSIDriver):
         # to set any that were provided
         params = {'volumeID': sf_volume_id}
 
+        create_time = timeutils.strtime(v_ref['created_at'])
         attributes = {'uuid': v_ref['id'],
                       'is_clone': 'True',
-                      'src_uuid': src_uuid}
+                      'src_uuid': src_uuid,
+                      'created_at': create_time}
         if qos:
             params['qos'] = qos
             for k, v in qos.items():
@@ -481,11 +498,13 @@ class SolidFire(SanISCSIDriver):
         if type_id is not None:
             qos = self._set_qos_by_volume_type(ctxt, type_id)
 
+        create_time = timeutils.strtime(volume['created_at'])
         attributes = {'uuid': volume['id'],
-                      'is_clone': 'False'}
+                      'is_clone': 'False',
+                      'created_at': create_time}
         if qos:
             for k, v in qos.items():
-                            attributes[k] = str(v)
+                attributes[k] = str(v)
 
         params = {'name': 'UUID-%s' % volume['id'],
                   'accountID': None,
@@ -600,6 +619,32 @@ class SolidFire(SanISCSIDriver):
 
         return self.cluster_stats
 
+    def extend_volume(self, volume, new_size):
+        """Extend an existing volume."""
+        LOG.debug(_("Entering SolidFire extend_volume..."))
+
+        sfaccount = self._get_sfaccount(volume['project_id'])
+        params = {'accountID': sfaccount['accountID']}
+
+        sf_vol = self._get_sf_volume(volume['id'], params)
+
+        if sf_vol is None:
+            LOG.error(_("Volume ID %s was not found on "
+                        "the SolidFire Cluster!"), volume['id'])
+            raise exception.VolumeNotFound(volume_id=volume['id'])
+
+        params = {
+            'volumeID': sf_vol['volumeID'],
+            'totalSize': int(new_size * self.GB)
+        }
+        data = self._issue_api_request('ModifyVolume',
+                                       params, version='5.0')
+
+        if 'result' not in data:
+            raise exception.SolidFireAPIDataException(data=data)
+
+        LOG.debug(_("Leaving SolidFire extend_volume"))
+
     def _update_cluster_status(self):
         """Retrieve status info for the Cluster."""
 
@@ -621,7 +666,7 @@ class SolidFire(SanISCSIDriver):
         backend_name = self.configuration.safe_get('volume_backend_name')
         data["volume_backend_name"] = backend_name or self.__class__.__name__
         data["vendor_name"] = 'SolidFire Inc'
-        data["driver_version"] = VERSION
+        data["driver_version"] = self.VERSION
         data["storage_protocol"] = 'iSCSI'
 
         data['total_capacity_gb'] = results['maxProvisionedSpace']
@@ -636,3 +681,79 @@ class SolidFire(SanISCSIDriver):
         data['thin_provision_percent'] =\
             results['thinProvisioningPercent']
         self.cluster_stats = data
+
+    def attach_volume(self, context, volume,
+                      instance_uuid, host_name,
+                      mountpoint):
+
+        LOG.debug(_("Entering SolidFire attach_volume..."))
+        sfaccount = self._get_sfaccount(volume['project_id'])
+        params = {'accountID': sfaccount['accountID']}
+
+        sf_vol = self._get_sf_volume(volume['id'], params)
+        if sf_vol is None:
+            LOG.error(_("Volume ID %s was not found on "
+                        "the SolidFire Cluster!"), volume['id'])
+            raise exception.VolumeNotFound(volume_id=volume['id'])
+
+        attributes = sf_vol['attributes']
+        attributes['attach_time'] = volume.get('attach_time', None)
+        attributes['attached_to'] = instance_uuid
+        params = {
+            'volumeID': sf_vol['volumeID'],
+            'attributes': attributes
+        }
+
+        data = self._issue_api_request('ModifyVolume', params)
+
+        if 'result' not in data:
+            raise exception.SolidFireAPIDataException(data=data)
+
+    def detach_volume(self, context, volume):
+
+        LOG.debug(_("Entering SolidFire attach_volume..."))
+        sfaccount = self._get_sfaccount(volume['project_id'])
+        params = {'accountID': sfaccount['accountID']}
+
+        sf_vol = self._get_sf_volume(volume['id'], params)
+        if sf_vol is None:
+            LOG.error(_("Volume ID %s was not found on "
+                        "the SolidFire Cluster!"), volume['id'])
+            raise exception.VolumeNotFound(volume_id=volume['id'])
+
+        attributes = sf_vol['attributes']
+        attributes['attach_time'] = None
+        attributes['attached_to'] = None
+        params = {
+            'volumeID': sf_vol['volumeID'],
+            'attributes': attributes
+        }
+
+        data = self._issue_api_request('ModifyVolume', params)
+
+        if 'result' not in data:
+            raise exception.SolidFireAPIDataException(data=data)
+
+    def accept_transfer(self, context, volume,
+                        new_user, new_project):
+
+        sfaccount = self._get_sfaccount(volume['project_id'])
+        params = {'accountID': sfaccount['accountID']}
+        sf_vol = self._get_sf_volume(volume['id'], params)
+
+        if new_project != volume['project_id']:
+            # do a create_sfaccount here as this tenant
+            # may not exist on the cluster yet
+            sfaccount = self._create_sfaccount(new_project)
+
+        params = {
+            'volumeID': sf_vol['volumeID'],
+            'accountID': sfaccount['accountID']
+        }
+        data = self._issue_api_request('ModifyVolume',
+                                       params, version='5.0')
+
+        if 'result' not in data:
+            raise exception.SolidFireAPIDataException(data=data)
+
+        LOG.debug(_("Leaving SolidFire transfer volume"))
