@@ -20,15 +20,27 @@ import sys
 import install_venv_common as install_venv
 
 
+def first_file(file_list):
+    for candidate in file_list:
+        if os.path.exists(candidate):
+            return candidate
+
+
 def main(argv):
     root = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 
     venv = os.environ['VIRTUAL_ENV']
 
-    pip_requires = os.path.join(root, 'tools', 'pip-requires')
-    test_requires = os.path.join(root, 'tools', 'test-requires')
+    pip_requires = first_file([
+        os.path.join(root, 'requirements.txt'),
+        os.path.join(root, 'tools', 'pip-requires'),
+    ])
+    test_requires = first_file([
+        os.path.join(root, 'test-requirements.txt'),
+        os.path.join(root, 'tools', 'test-requires'),
+    ])
     py_version = "python%s.%s" % (sys.version_info[0], sys.version_info[1])
-    project = 'Cinder'
+    project = 'cinder'
     install = install_venv.InstallVenv(root, venv, pip_requires, test_requires,
                                        py_version, project)
     #NOTE(dprince): For Tox we only run post_process (which patches files, etc)
